/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration;

import java.util.function.Consumer;
import java.util.function.Predicate;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.context.event.EventListener;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.stereotype.Component;

import de.ozgcloud.collaboration.common.callcontext.CurrentUserService;
import de.ozgcloud.collaboration.fachstelle.FachstelleService;
import de.ozgcloud.command.Command;
import de.ozgcloud.command.CommandCreatedEvent;
import de.ozgcloud.command.CommandFailedEvent;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Component
@RequiredArgsConstructor
class CollaborationEventListener {

	public static final String CREATE_COLLABORATION_REQUEST_ORDER = "CREATE_COLLABORATION_REQUEST";
	public static final Predicate<Command> IS_CREATE_COLLABORATION_REQUEST_COMMAND = command -> CREATE_COLLABORATION_REQUEST_ORDER.equals(
			command.getOrder());
	private static final String IS_CREATE_COLLABORATION_REQUEST = "{T(de.ozgcloud.collaboration.CollaborationEventListener).IS_CREATE_COLLABORATION_REQUEST_COMMAND.test(event.getSource())}";

	private static final String LOG_MESSAGE_TEMPLATE = "{}. Command failed.";
	private static final String ERROR_MESSAGE_TEMPLATE = "Error on executing %s Command (id: %s).";

	private final CollaborationService collaborationService;
	private final FachstelleService fachstelleService;
	@Qualifier(CollaborationManagerConfiguration.CURRENT_USER_SERVICE_NAME) // NOSONAR
	private final CurrentUserService currentUserService;
	private final ApplicationEventPublisher eventPublisher;
	@Qualifier(CollaborationManagerConfiguration.COLLABORATION_REQUEST_MAPPER_NAME) // NOSONAR
	private final CollaborationRequestMapper collaborationRequestMapper;

	@EventListener(condition = IS_CREATE_COLLABORATION_REQUEST)
	public void onCreateCollaborationRequest(CommandCreatedEvent event) {
		runWithSecurityContext(event.getSource(), this::createCollaborationRequest);
	}

	void createCollaborationRequest(Command command) {
		var collaborationRequest = collaborationRequestMapper.toCollaborationRequestExcludingFachstelle(command);
		switch (collaborationRequest.getCollaborationLevel()) {
			case 1 -> createLevel1CollaborationRequest(collaborationRequest, command);
			case 4 -> createLevel4CollaborationRequest(collaborationRequest, command);
			default -> {
				LOG.error("Unknown Collaboration Level: {}", collaborationRequest.getCollaborationLevel());
				eventPublisher.publishEvent(new CommandFailedEvent(command.getId(), "Unknown Collaboration Level."));
			}
		}
	}

	void createLevel1CollaborationRequest(CollaborationRequest collaborationRequest, Command command) {
		var fachstelle = fachstelleService
				.getOrganisationEinheit(MapUtils.getString(command.getBodyObject(), CollaborationRequest.PROPERTY_ZUSTAENDIGE_STELLE));
		collaborationService.createCollaborationRequest(collaborationRequest.toBuilder().zustaendigeStelle(fachstelle).build());
	}

	void createLevel4CollaborationRequest(CollaborationRequest collaborationRequest, Command command) {
		var fachstelle = fachstelleService
				.getExterneFachstelle(MapUtils.getString(command.getBodyObject(), CollaborationRequest.PROPERTY_ZUSTAENDIGE_STELLE));
		collaborationService.createFachstellenBeteiligungRequest(collaborationRequest.toBuilder().zustaendigeStelle(fachstelle).build());
	}

	void runWithSecurityContext(Command command, Consumer<Command> commandExecutor) {
		SecurityContext prevContext = null;
		try {
			prevContext = currentUserService.startSecurityContext(command);
			commandExecutor.accept(command);
		} catch (Exception e) {
			var errorMessage = ERROR_MESSAGE_TEMPLATE.formatted(command.getOrder(), command.getId());
			LOG.error(LOG_MESSAGE_TEMPLATE, errorMessage, e);
			eventPublisher.publishEvent(new CommandFailedEvent(command.getId(), buildErrorMessage(errorMessage, e)));
		} finally {
			currentUserService.resetSecurityContext(prevContext);
		}
	}

	private String buildErrorMessage(String message, Exception cause) {
		try {
			StringBuilder sb = new StringBuilder(message);
			if (StringUtils.isNotBlank(cause.getMessage())) {
				sb.append(" Cause: ").append(cause.getMessage());
			}
			return sb.toString();
		} catch (Exception e2) {
			LOG.error("Error in building Error Message (sick).", e2);
			return message;
		}
	}

}
