/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import de.ozgcloud.apilib.common.command.OzgCloudCommandService;
import de.ozgcloud.apilib.common.command.grpc.CommandMapper;
import de.ozgcloud.apilib.common.command.grpc.GrpcOzgCloudCommandService;
import de.ozgcloud.apilib.file.OzgCloudFileService;
import de.ozgcloud.apilib.file.grpc.GrpcOzgCloudFileService;
import de.ozgcloud.apilib.file.grpc.OzgCloudFileMapper;
import de.ozgcloud.apilib.user.GrpcOzgCloudUserProfileService;
import de.ozgcloud.apilib.user.OzgCloudUserProfileService;
import de.ozgcloud.apilib.user.UserProfileMapper;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangService;
import de.ozgcloud.apilib.vorgang.grpc.GrpcOzgCloudVorgangService;
import de.ozgcloud.apilib.vorgang.grpc.OzgCloudVorgangMapper;
import de.ozgcloud.apilib.vorgang.grpc.OzgCloudVorgangQueryMapper;
import de.ozgcloud.apilib.vorgang.grpc.OzgCloudVorgangStubMapper;
import de.ozgcloud.collaboration.common.callcontext.CollaborationManagerCallContextProvider;
import de.ozgcloud.user.grpc.userprofile.UserProfileServiceGrpc.UserProfileServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.binaryFile.BinaryFileServiceGrpc.BinaryFileServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.binaryFile.BinaryFileServiceGrpc.BinaryFileServiceStub;
import de.ozgcloud.vorgang.grpc.command.CommandServiceGrpc.CommandServiceBlockingStub;
import de.ozgcloud.vorgang.vorgang.VorgangServiceGrpc.VorgangServiceBlockingStub;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Configuration
public class CollaborationManagerConfiguration {

	public static final String COMMAND_SERVICE_NAME = "collaboration_OzgCloudCommandService";
	public static final String FILE_SERVICE_NAME = "collaboration_OzgCloudFileService";
	public static final String CURRENT_USER_SERVICE_NAME = "collaboration_CurrentUserService";
	public static final String OZGCLOUD_USER_PROFILE_SERVICE_NAME = "collaboration_OzgCloudUserProfileService";
	public static final String USER_PROFILE_SERVICE_NAME = "collaboration_UserProfileService";
	public static final String VORGANG_SERVICE_NAME = "collaboration_VorgangService";
	public static final String VORGANG_REMOTE_SERVICE_NAME = "collaboration_VorgangRemoteService";
	public static final String VORGANG_OZGCLOUD_SERVICE_NAME = "collaboration_OzgCloudVorgangService";
	public static final String COLLABORATION_SERVICE_NAME = "collaboration_CollaborationService";
	public static final String COLLABORATION_REQUEST_MAPPER_NAME = "collaboration_CollaborationRequestMapper";
	public static final String FACHSTELLE_MAPPER_NAME = "collaboration_FachstelleMapper";
	public static final String TEMPLATE_HANDLER_NAME = "collaboration_TemplateHandler";
	public static final String VORGANG_ATTACHED_ITEM_REMOTE_SERVICE_NAME = "collaboration_VorgangAttachedItemRemoteService";
	public static final String VORGANG_ATTACHED_ITEM_SERVICE_NAME = "collaboration_VorgangAttachedItemService";
	public static final String VORGANG_MANAGER_PROPERTIES_NAME = "collaboration_vorgangManagerProperties";

	public static final String CALL_CONTEXT_INTERCEPTOR_NAME = "collaboration_CallContextInterceptor";
	public static final String CALL_CONTEXT_PROVIDER_NAME = "collaboration_CallContextProvider";

	public static final String GRPC_VORGANG_MANAGER_NAME = "vorgang-manager";
	public static final String GRPC_COMMAND_MANAGER_NAME = "command-manager";
	public static final String GRPC_FILE_MANAGER_NAME = "file-manager";
	public static final String GRPC_USER_MANAGER_NAME = "user-manager";
	public static final String GRPC_ZUFI_MANAGER_NAME = "zufi-manager";

	@GrpcClient(GRPC_COMMAND_MANAGER_NAME)
	private CommandServiceBlockingStub commandServiceStub;

	@GrpcClient(GRPC_VORGANG_MANAGER_NAME)
	private VorgangServiceBlockingStub vorgangServiceStub;

	@GrpcClient(GRPC_FILE_MANAGER_NAME)
	private BinaryFileServiceBlockingStub binaryFileServiceStub;
	@GrpcClient(GRPC_FILE_MANAGER_NAME)
	private BinaryFileServiceStub binaryFileServiceAsyncStub;

	@GrpcClient(GRPC_USER_MANAGER_NAME)
	private UserProfileServiceBlockingStub userProfileServiceStub;

	@Bean(COMMAND_SERVICE_NAME) // NOSONAR
	OzgCloudCommandService grpcOzgCloudCommandService(CommandMapper commandMapper, CollaborationManagerCallContextProvider contextProvider) {
		return new GrpcOzgCloudCommandService(commandServiceStub, commandMapper, contextProvider,
				GrpcOzgCloudCommandService.DEFAULT_COMMAND_REQUEST_THRESHOLD_MILLIS);
	}

	@Bean(FILE_SERVICE_NAME) // NOSONAR
	OzgCloudFileService grpcOzgClouFileService(OzgCloudFileMapper mapper, CollaborationManagerCallContextProvider contextProvider) {
		return new GrpcOzgCloudFileService(binaryFileServiceStub, binaryFileServiceAsyncStub, contextProvider, mapper);
	}

	@Bean(VORGANG_OZGCLOUD_SERVICE_NAME) // NOSONAR
	OzgCloudVorgangService grpcOzgCloudVorgangService(OzgCloudVorgangMapper vorgangMapper, OzgCloudVorgangStubMapper vorgangStubMapper,
			OzgCloudVorgangQueryMapper vorgangQueryMapper, CollaborationManagerCallContextProvider contextProvider) {
		return new GrpcOzgCloudVorgangService(vorgangServiceStub, vorgangMapper, vorgangStubMapper, vorgangQueryMapper, contextProvider);
	}

	@Bean(OZGCLOUD_USER_PROFILE_SERVICE_NAME) // NOSONAR
	OzgCloudUserProfileService grpcOzgCloudUserProfileService(UserProfileMapper userProfileMapper,
			CollaborationManagerCallContextProvider contextProvider) {
		return new GrpcOzgCloudUserProfileService(userProfileServiceStub, userProfileMapper, contextProvider);
	}
}