/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration;

import java.util.Map;

import org.apache.commons.collections.MapUtils;
import org.mapstruct.AnnotateWith;
import org.mapstruct.CollectionMappingStrategy;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.NullValuePropertyMappingStrategy;
import org.mapstruct.ReportingPolicy;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.ozgcloud.apilib.common.command.OzgCloudCommand;
import de.ozgcloud.apilib.common.command.grpc.CommandMapper;
import de.ozgcloud.collaboration.common.callcontext.CollaborationManagerCallContextGrpcClientInterceptor;
import de.ozgcloud.collaboration.common.vorgang.attached_item.VorgangAttachedItem;
import de.ozgcloud.collaboration.fachstelle.Fachstelle;
import de.ozgcloud.collaboration.fachstelle.FachstelleMapper;
import de.ozgcloud.collaboration.request.CollaborationRequestId;
import de.ozgcloud.collaboration.request.GrpcCollaborationRequest;
import de.ozgcloud.command.Command;
import de.ozgcloud.vorgang.vorgang.GrpcCreateCollaborationRequestData;

@AnnotateWith(value = Component.class, elements = @AnnotateWith.Element(strings = CollaborationManagerConfiguration.COLLABORATION_REQUEST_MAPPER_NAME))
@Mapper(uses = { CommandMapper.class }, //
		unmappedTargetPolicy = ReportingPolicy.WARN, //
		unmappedSourcePolicy = ReportingPolicy.WARN, //
		nullValuePropertyMappingStrategy = NullValuePropertyMappingStrategy.IGNORE, //
		nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS, //
		collectionMappingStrategy = CollectionMappingStrategy.ADDER_PREFERRED)
public abstract class CollaborationRequestMapper {

	static final String ITEM_NAME = "CollaborationRequest";

	static final String FIELD_ITEM_CLIENT = "client";
	static final String FIELD_ITEM_VORGANG_ID = "vorgangId";
	static final String FIELD_ITEM_NAME = "itemName";
	static final String FIELD_ITEM_BODY = "item";

	@Autowired
	private FachstelleMapper fachstelleMapper;

	@Mapping(target = "collaborationVorgangId", ignore = true)
	@Mapping(target = "createdBy", ignore = true)
	@Mapping(target = "zustaendigeStelle", ignore = true)
	@Mapping(target = "commandId", source = "id")
	@Mapping(target = "titel", expression = "java(getStringProperty(CollaborationRequest.PROPERTY_TITEL, command.getBodyObject()))")
	@Mapping(target = "beschreibung", expression = "java(getStringProperty(CollaborationRequest.PROPERTY_BESCHREIBUNG, command.getBodyObject()))")
	@Mapping(target = "collaborationLevel", expression = "java(getIntProperty(CollaborationRequest.PROPERTY_COLLABORATION_LEVEL, command.getBodyObject()))")
	public abstract CollaborationRequest toCollaborationRequestExcludingFachstelle(Command command);

	@Mapping(target = "id", ignore = true)
	@Mapping(target = "status", ignore = true)
	@Mapping(target = "relationVersion", ignore = true)
	@Mapping(target = "finishedAt", ignore = true)
	@Mapping(target = "errorMessage", ignore = true)
	@Mapping(target = "createdResource", ignore = true)
	@Mapping(target = "createdAt", ignore = true)
	@Mapping(target = "body", ignore = true)

	@Mapping(target = "vorgangId", source = "request.vorgangId")
	@Mapping(target = "relationId", expression = "java(commandMapper.mapRelationId(request.getVorgangId()))")
	@Mapping(target = "bodyObject", expression = "java(toBodyObject(request))")
	public abstract OzgCloudCommand toOzgCloudCommand(CollaborationRequest request, String order);

	Map<String, Object> toBodyObject(CollaborationRequest request) {
		return Map.of(
				FIELD_ITEM_VORGANG_ID, request.getVorgangId(),
				FIELD_ITEM_CLIENT, CollaborationManagerCallContextGrpcClientInterceptor.COLLABORATION_MANAGER_CLIENT_NAME,
				FIELD_ITEM_NAME, ITEM_NAME,
				FIELD_ITEM_BODY, toItemMap(request));
	}

	Map<String, Object> toItemMap(CollaborationRequest request) {
		return Map.of(
				CollaborationRequest.PROPERTY_TITEL, request.getTitel(),
				CollaborationRequest.PROPERTY_BESCHREIBUNG, request.getBeschreibung(),
				CollaborationRequest.PROPERTY_ZUSTAENDIGE_STELLE, fachstelleMapper.toMap(request.getZustaendigeStelle()),
				CollaborationRequest.PROPERTY_COLLABORATION_LEVEL, request.getCollaborationLevel(),
				CollaborationRequest.PROPERTY_COLLABORATION_VORGANG_ID, request.getCollaborationVorgangId());
	}

	@Mapping(target = "titel", source = "titel")
	@Mapping(target = "zustaendigeStelle", source = "zustaendigeStelle.technicalId")
	@Mapping(target = "idBytes", ignore = true)
	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "beschreibungBytes", ignore = true)
	@Mapping(target = "collaborationVorgangIdBytes", ignore = true)
	@Mapping(target = "createdByBytes", ignore = true)
	@Mapping(target = "titelBytes", ignore = true)
	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "zustaendigeStelleBytes", ignore = true)
	@Mapping(target = "allFields", ignore = true)
	public abstract GrpcCollaborationRequest toGrpcCollaborationRequest(CollaborationRequest request);

	@Mapping(target = "organisationEinheitId", source = "zustaendigeStelle.subjectId")
	@Mapping(target = "organisationEinheitIdBytes", ignore = true)
	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "vorgangIdBytes", ignore = true)
	@Mapping(target = "allFields", ignore = true)
	public abstract GrpcCreateCollaborationRequestData toGrpcCreateCollaborationRequestData(CollaborationRequest request);

	@Mapping(target = "id", expression = "java(toCollaborationRequestId(getStringProperty(CollaborationRequest.PROPERTY_ID, item.getItem())))")
	@Mapping(target = "collaborationLevel", expression = "java(getIntProperty(CollaborationRequest.PROPERTY_COLLABORATION_LEVEL, item.getItem()))")
	@Mapping(target = "vorgangId", expression = "java(getStringProperty(CollaborationRequest.PROPERTY_VORGANG_ID, item.getItem()))")
	@Mapping(target = "collaborationVorgangId", expression = "java(getStringProperty(CollaborationRequest.PROPERTY_COLLABORATION_VORGANG_ID, item.getItem()))")
	@Mapping(target = "commandId", expression = "java(getStringProperty(CollaborationRequest.PROPERTY_COMMAND_ID, item.getItem()))")
	@Mapping(target = "createdBy", expression = "java(getStringProperty(CollaborationRequest.PROPERTY_CREATED_BY, item.getItem()))")
	@Mapping(target = "beschreibung", expression = "java(getStringProperty(CollaborationRequest.PROPERTY_BESCHREIBUNG, item.getItem()))")
	@Mapping(target = "titel", expression = "java(getStringProperty(CollaborationRequest.PROPERTY_TITEL, item.getItem()))")
	@Mapping(target = "zustaendigeStelle", expression = "java(getZustaendigeStelleFromItemMap(item.getItem()))")
	public abstract CollaborationRequest mapFromVorgangAttachedItem(VorgangAttachedItem item);

	CollaborationRequestId toCollaborationRequestId(String id) {
		return CollaborationRequestId.from(id);
	}

	Fachstelle getZustaendigeStelleFromItemMap(Map<String, Object> propertyMap) {
		return fachstelleMapper.fromMap(MapUtils.getMap(propertyMap, CollaborationRequest.PROPERTY_ZUSTAENDIGE_STELLE));
	}

	int getIntProperty(String key, Map<String, Object> propertyMap) {
		return MapUtils.getIntValue(propertyMap, key);
	}

	String getStringProperty(String key, Map<String, Object> propertyMap) {
		return MapUtils.getString(propertyMap, key);
	}
}
