/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration;

import java.util.List;
import java.util.Map;
import java.util.Optional;

import jakarta.validation.Valid;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import de.ozgcloud.apilib.common.command.OzgCloudCommand;
import de.ozgcloud.apilib.common.command.OzgCloudCommandService;
import de.ozgcloud.apilib.common.command.OzgCloudCreateSubCommandsRequest;
import de.ozgcloud.apilib.common.command.grpc.CommandMapper;
import de.ozgcloud.apilib.file.OzgCloudFileService;
import de.ozgcloud.collaboration.common.freemarker.TemplateHandler;
import de.ozgcloud.collaboration.common.user.UserProfileService;
import de.ozgcloud.collaboration.vorgang.ServiceKonto;
import de.ozgcloud.collaboration.vorgang.VorgangService;
import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.RequiredArgsConstructor;

@Service(CollaborationManagerConfiguration.COLLABORATION_SERVICE_NAME) // NOSONAR
@RequiredArgsConstructor
@Validated
public class CollaborationService {

	static final String CREATE_ATTACHED_ITEM_ORDER = "CREATE_ATTACHED_ITEM";
	static final String SUBCOMMANDS_EXECUTION_MODE = "PARALLEL";
	public static final String KEY_VORGANG_LINK = "linkToVorgang";

	public static final String COLLABORATION_NACHRICHT_TEMPLATE = "collaboration.nachrichtTemplate.txt.ftlh";
	static final String SEND_POSTFACH_NACHRICHT_ORDER = "SEND_POSTFACH_NACHRICHT";
	static final String FIELD_REPLY_OPTION = "replyOption";
	static final String REPLY_OPTION = "FORBIDDEN";
	static final String FIELD_SUBJECT = "subject";
	static final String FIELD_MAIL_BODY = "mailBody";
	static final String SUBJECT = "Kooperationsanfrage";
	static final String MUK_SERVICEKONTO_TYPE = "MUK";
	static final String MUK_POSTFACH_ADDRESS_VERSION = "1.0";
	static final int MUK_POSTFACH_ADDRESS_TYPE = 1;

	@Qualifier(CollaborationManagerConfiguration.VORGANG_SERVICE_NAME) // NOSONAR
	private final VorgangService vorgangService;

	@Qualifier(CollaborationManagerConfiguration.COMMAND_SERVICE_NAME) // NOSONAR
	private final OzgCloudCommandService commandService;

	@Qualifier(CollaborationManagerConfiguration.USER_PROFILE_SERVICE_NAME) // NOSONAR
	private final UserProfileService userProfileService;

	@Qualifier(CollaborationManagerConfiguration.FILE_SERVICE_NAME) // NOSONAR
	private final OzgCloudFileService fileService;

	@Qualifier(CollaborationManagerConfiguration.COLLABORATION_REQUEST_MAPPER_NAME) // NOSONAR
	private final CollaborationRequestMapper collaborationRequestMapper;
	private final CommandMapper commandMapper;
	@Qualifier(CollaborationManagerConfiguration.TEMPLATE_HANDLER_NAME) // NOSONAR
	private final TemplateHandler templateHandler;
	private final Optional<FachstelleVorgangUrlProvider> urlProvider;

	public void createCollaborationRequest(@Valid CollaborationRequest collaborationRequest) {
		var enrichedRequest = enrichCollaborationRequest(collaborationRequest, vorgangService.createCollaborationVorgang(collaborationRequest));
		commandService.addSubCommands(buildCreateCollaborationRequestRequest(enrichedRequest));
	}

	OzgCloudCreateSubCommandsRequest buildCreateCollaborationRequestRequest(CollaborationRequest request) {
		return OzgCloudCreateSubCommandsRequest.builder()
				.parentId(request.getCommandId())
				.executionMode(SUBCOMMANDS_EXECUTION_MODE)
				.completedIfSubsCompleted(true)
				.subCommand(collaborationRequestMapper.toOzgCloudCommand(request, CREATE_ATTACHED_ITEM_ORDER))
				.build();
	}

	public void createFachstellenBeteiligungRequest(@Valid CollaborationRequest collaborationRequest) {
		if (urlProvider.isEmpty()) {
			throw new TechnicalException("Fachstelle not configured.");
		}
		var vorgang = vorgangService.getVorgang(collaborationRequest.getVorgangId());
		var enrichedRequest = enrichCollaborationRequest(collaborationRequest, vorgang.getId().toString());
		var addSubCommandsRequest = buildCreateSubCommandsRequest(enrichedRequest, buildSubCommands(enrichedRequest));
		commandService.addSubCommands(addSubCommandsRequest);
	}

	CollaborationRequest enrichCollaborationRequest(CollaborationRequest request, String collaborationVorgangId) {
		return request.toBuilder().collaborationVorgangId(collaborationVorgangId).createdBy(getUserId()).build();
	}

	String getUserId() {
		return userProfileService.getUserProfile().getId().toString();
	}

	OzgCloudCreateSubCommandsRequest buildCreateSubCommandsRequest(CollaborationRequest collaborationRequest, List<OzgCloudCommand> subCommands) {
		return OzgCloudCreateSubCommandsRequest.builder()
				.parentId(collaborationRequest.getCommandId())
				.executionMode(SUBCOMMANDS_EXECUTION_MODE)
				.completedIfSubsCompleted(true)
				.subCommands(subCommands)
				.build();
	}

	List<OzgCloudCommand> buildSubCommands(CollaborationRequest collaborationRequest) {
		return List.of(
				collaborationRequestMapper.toOzgCloudCommand(collaborationRequest, CREATE_ATTACHED_ITEM_ORDER),
				buildSendPostfachNachrichtCommand(collaborationRequest));
	}

	OzgCloudCommand buildSendPostfachNachrichtCommand(CollaborationRequest collaborationRequest) {
		var bodyObject = buildPostfachSendNachrichtCommandBody(buildMukPostfachAddress(collaborationRequest.getZustaendigeStelle().getSubjectId()),
				collaborationRequest.getVorgangId());
		return OzgCloudCommand.builder()
				.vorgangId(commandMapper.toOzgCloudVorgangId(collaborationRequest.getVorgangId()))
				.relationId(commandMapper.mapRelationId(collaborationRequest.getVorgangId()))
				.order(SEND_POSTFACH_NACHRICHT_ORDER)
				.createdBy(commandMapper.toOzgCloudUserId(collaborationRequest.getCreatedBy()))
				.bodyObject(bodyObject)
				.build();
	}

	Map<String, Object> buildMukPostfachAddress(String postfachId) {
		return Map.of(
				ServiceKonto.PostfachAddress.FIELD_TYPE, MUK_POSTFACH_ADDRESS_TYPE,
				ServiceKonto.PostfachAddress.FIELD_VERSION, MUK_POSTFACH_ADDRESS_VERSION,
				ServiceKonto.PostfachAddress.FIELD_IDENTIFIER, buildPostfachIdentifier(postfachId),
				ServiceKonto.FIELD_SERVICEKONTO_TYPE, MUK_SERVICEKONTO_TYPE);
	}

	Map<String, Object> buildPostfachIdentifier(String postfachId) {
		return Map.of(ServiceKonto.PostfachAddress.FIELD_IDENTIFIER_POSTFACH_ID, postfachId);
	}

	Map<String, Object> buildPostfachSendNachrichtCommandBody(Map<String, Object> postfachAddress, String vorgangId) {
		return Map.of(
				FIELD_REPLY_OPTION, REPLY_OPTION,
				FIELD_SUBJECT, SUBJECT,
				FIELD_MAIL_BODY, getAnfrageText(vorgangId),
				ServiceKonto.FIELD_POSTFACH_ADDRESS, postfachAddress);
	}

	String getAnfrageText(String vorgangId) {
		return templateHandler.fillTemplate(COLLABORATION_NACHRICHT_TEMPLATE, getTemplateParameters(vorgangId));
	}

	Map<String, Object> getTemplateParameters(String vorgangId) {
		return Map.of(KEY_VORGANG_LINK, urlProvider.get().getUrl(vorgangId));
	}
}