/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration.common.callcontext;

import static de.ozgcloud.common.grpc.GrpcUtil.*;

import java.util.UUID;

import org.apache.logging.log4j.ThreadContext;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import de.ozgcloud.collaboration.CollaborationManagerConfiguration;
import io.grpc.CallOptions;
import io.grpc.Channel;
import io.grpc.ClientCall;
import io.grpc.ClientInterceptor;
import io.grpc.ForwardingClientCall.SimpleForwardingClientCall;
import io.grpc.Metadata;
import io.grpc.Metadata.Key;
import io.grpc.MethodDescriptor;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Component(CollaborationManagerConfiguration.CALL_CONTEXT_INTERCEPTOR_NAME)
@RequiredArgsConstructor
public class CollaborationManagerCallContextGrpcClientInterceptor implements ClientInterceptor {

	public static final String COLLABORATION_MANAGER_CLIENT_NAME = "OzgCloud_CollaborationManager";

	private static final String REQUEST_ID_KEY = "requestId";
	// TODO In GrpcUtil verschieben
	public static final Key<byte[]> KEY_ACCESS_LIMITED = createKeyOf("ACCESS_LIMITED-bin");
	public static final Key<byte[]> KEY_ACCESS_LIMITED_ORGAID = createKeyOf("ACCESS_LIMITED_TO_ORGANISATORISCHEEINHEITENID-bin");

	@Qualifier(CollaborationManagerConfiguration.CURRENT_USER_SERVICE_NAME)
	private final CurrentUserService userService;

	// <A> = Request, <B> = Response
	@Override
	public <A, B> ClientCall<A, B> interceptCall(MethodDescriptor<A, B> method, CallOptions callOptions, Channel next) {
		return new CallContextAttachingClientCall<>(next.newCall(method, callOptions));
	}

	final class CallContextAttachingClientCall<A, B> extends SimpleForwardingClientCall<A, B> {

		private CallContextAttachingClientCall(ClientCall<A, B> delegate) {
			super(delegate);
		}

		@Override
		public void start(Listener<B> responseListener, Metadata headers) {
			headers.merge(buildCallContextMetadata());
			super.start(responseListener, headers);
		}

		private Metadata buildCallContextMetadata() {
			var metadata = new Metadata();

			var user = userService.getUser();
			user.getUserId().ifPresent(userId -> metadata.put(HEADER_KEY_USER_ID, userId.getBytes()));
			metadata.put(KEY_ACCESS_LIMITED, String.valueOf(user.isOrganisationEinheitenIdCheckNecessary()).getBytes());
			user.getOrganisatorischeEinheitenIds().forEach(id -> metadata.put(KEY_ACCESS_LIMITED_ORGAID, id.getBytes()));
			metadata.put(HEADER_KEY_CLIENT_NAME, COLLABORATION_MANAGER_CLIENT_NAME.getBytes());
			metadata.put(HEADER_KEY_REQUEST_ID, getRequestId());

			return metadata;
		}

		private byte[] getRequestId() {
			return ThreadContext.getContext().getOrDefault(REQUEST_ID_KEY, UUID.randomUUID().toString()).getBytes();
		}
	}
}