/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration.common.callcontext;

import java.util.UUID;

import org.apache.logging.log4j.CloseableThreadContext;
import org.springframework.security.core.context.SecurityContextHolder;

import de.ozgcloud.common.grpc.GrpcUtil;
import io.grpc.ForwardingServerCallListener;
import io.grpc.Metadata;
import io.grpc.ServerCall;
import io.grpc.ServerCall.Listener;
import io.grpc.ServerCallHandler;
import io.grpc.ServerInterceptor;
import lombok.extern.log4j.Log4j2;

@Log4j2
public class CollaborationManagerCallContextGrpcServerInterceptor implements ServerInterceptor {

	@Override
	public <A, B> Listener<A> interceptCall(ServerCall<A, B> call, Metadata headers, ServerCallHandler<A, B> next) {
		return new LogContextSettingListener<>(next.startCall(call, headers), headers);
	}

	class LogContextSettingListener<A> extends ForwardingServerCallListener.SimpleForwardingServerCallListener<A> {
		private final String requestId;
		private final Metadata headers;

		public LogContextSettingListener(ServerCall.Listener<A> delegate, Metadata headers) {
			super(delegate);
			this.headers = headers;
			this.requestId = getRequestId();
		}

		@Override
		public void onMessage(A message) {
			doSurroundOn(() -> super.onMessage(message));
		}

		@Override
		public void onHalfClose() {
			doSurroundOn(super::onHalfClose);
		}

		@Override
		public void onCancel() {
			doSurroundOn(super::onCancel);
		}

		@Override
		public void onComplete() {
			doSurroundOn(super::onComplete);
		}

		@Override
		public void onReady() {
			doSurroundOn(super::onReady);
		}

		void doSurroundOn(Runnable runnable) {
			try (var ctc = CloseableThreadContext.put(GrpcUtil.KEY_REQUEST_ID, requestId)) {
				startSecurityContext();
				runnable.run();
			} finally {
				clearSecurityContext();
			}
		}

		String getRequestId() {
			return GrpcUtil.getRequestId(headers).orElseGet(() -> UUID.randomUUID().toString());
		}

		void startSecurityContext() {
			var secContext = SecurityContextHolder.getContext();
			secContext.setAuthentication(CallContextAuthenticationToken.authenticate(createUser()));
		}

		CallContextUser createUser() {
			var builder = CallContextUser.builder()
					.userId(GrpcUtil.getUserId(headers))
					.userName(GrpcUtil.getUserName(headers));

			GrpcUtil.getAccessLimited(headers).ifPresentOrElse(
					isAccessLimited -> builder.organisationEinheitenIdCheckNecessary(isAccessLimited)
							.organisatorischeEinheitenIds(GrpcUtil.getAccessLimitedOrgaIds(headers)),
					() -> builder.organisationEinheitenIdCheckNecessary(false));

			// TODO throw exception if missing required data as soon all clients are fine
			// with using headers for auth data.
			GrpcUtil.getClientName(headers).ifPresentOrElse(builder::clientName, () -> LOG.warn("Missing client name in grpc header."));

			return builder.build();
		}

		void clearSecurityContext() {
			SecurityContextHolder.clearContext();
		}
	}
}