/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration.vorgang;

import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.stream.Stream;

import org.apache.commons.collections.CollectionUtils;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Named;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.ReportingPolicy;

import de.ozgcloud.apilib.vorgang.OzgCloudFormDataEntry;
import de.ozgcloud.apilib.vorgang.OzgCloudFormField;
import de.ozgcloud.apilib.vorgang.OzgCloudSubForm;
import de.ozgcloud.collaboration.GrpcFormData;
import de.ozgcloud.collaboration.GrpcFormField;
import de.ozgcloud.collaboration.GrpcSubForm;
import de.ozgcloud.collaboration.GrpcSubFormData;

@Mapper(unmappedTargetPolicy = ReportingPolicy.WARN, nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface CollaborationFormDataMapper {

	default List<FormDataEntry> fromOzgCloudEntries(List<OzgCloudFormDataEntry> entries) {
		if (CollectionUtils.isEmpty(entries)) {
			return Collections.emptyList();
		}
		return Stream.concat(fromOzgCloudFormFields(entries), fromOzgCloudSubForms(entries)).toList();
	}

	default Stream<FormDataEntry> fromOzgCloudFormFields(List<OzgCloudFormDataEntry> entries) {
		return Optional.ofNullable(entries).stream()
				.flatMap(List::stream)
				.filter(OzgCloudFormDataEntry::isFormEntry)
				.map(OzgCloudFormField.class::cast)
				.map(this::fromOzgCloudFormField);
	}

	default Stream<FormDataEntry> fromOzgCloudSubForms(List<OzgCloudFormDataEntry> entries) {
		return Optional.ofNullable(entries).stream()
				.flatMap(List::stream)
				.filter(OzgCloudFormDataEntry::isSubForm)
				.map(OzgCloudSubForm.class::cast)
				.map(this::fromOzgCloudSubForm);
	}

	default FormDataEntry fromOzgCloudSubForm(OzgCloudSubForm subForm) {
		var subForms = subForm.getFormEntrys().stream()
				.filter(OzgCloudFormDataEntry::isSubForm)
				.map(OzgCloudSubForm.class::cast)
				.map(this::fromOzgCloudSubForm);

		var formFields = subForm.getFormEntrys().stream()
				.filter(OzgCloudFormDataEntry::isFormEntry)
				.map(OzgCloudFormField.class::cast)
				.map(this::fromOzgCloudFormField);

		return SubForm.builder()
				.label(subForm.getLabel())
				.name(subForm.getName())
				.formEntrys(Stream.concat(subForms, formFields).toList())
				.build();
	}

	FormField fromOzgCloudFormField(OzgCloudFormField formField);

	default GrpcFormData toGrpcFormData(List<FormDataEntry> entries) {
		return GrpcFormData.newBuilder()
				.addAllFormData(Stream.concat(fromFormField(entries), fromSubForm(entries)).toList())
				.build();
	}

	default Stream<GrpcSubFormData> fromFormField(List<FormDataEntry> entries) {
		return Optional.ofNullable(entries).stream()
				.flatMap(List::stream)
				.filter(FormDataEntry::isFormEntry)
				.map(FormField.class::cast)
				.map(this::toGrpcFormField)
				.map(formField -> GrpcSubFormData.newBuilder().setField(formField).build());
	}

	default Stream<GrpcSubFormData> fromSubForm(List<FormDataEntry> entries) {
		return Optional.ofNullable(entries).stream()
				.flatMap(List::stream)
				.filter(FormDataEntry::isSubForm)
				.map(SubForm.class::cast)
				.map(this::toGrpcSubForm)
				.map(subForm -> GrpcSubFormData.newBuilder().setForm(subForm).build());
	}

	@Mapping(target = "value", qualifiedByName = "mapValue", source = "value")
	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "labelBytes", ignore = true)
	@Mapping(target = "nameBytes", ignore = true)
	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "valueBytes", ignore = true)
	@Mapping(target = "allFields", ignore = true)
	GrpcFormField toGrpcFormField(FormField formField);

	@Named("mapValue")
	default String toString(Object object) {
		return String.valueOf(object);
	}

	default GrpcSubForm toGrpcSubForm(SubForm subForm) {
		var subForms = subForm.getFormEntrys().stream()
				.filter(FormDataEntry::isSubForm)
				.map(SubForm.class::cast)
				.map(this::toGrpcSubForm)
				.map(entry -> GrpcSubFormData.newBuilder()
						.setForm(entry)
						.build());

		var formFields = subForm.getFormEntrys().stream()
				.filter(FormDataEntry::isFormEntry)
				.map(FormField.class::cast)
				.map(this::toGrpcFormField)
				.map(entry -> GrpcSubFormData.newBuilder()
						.setField(entry)
						.build());

		return GrpcSubForm.newBuilder()
				.setLabel(subForm.getLabel())
				.setName(subForm.getName())
				.addAllFormData(Stream.concat(subForms, formFields).toList())
				.build();
	}
}