/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Map;
import java.util.function.Consumer;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.security.core.context.SecurityContext;

import de.ozgcloud.collaboration.common.callcontext.CurrentUserService;
import de.ozgcloud.collaboration.fachstelle.FachstelleService;
import de.ozgcloud.collaboration.fachstelle.FachstelleTestFactory;
import de.ozgcloud.command.Command;
import de.ozgcloud.command.CommandCreatedEventTestFactory;
import de.ozgcloud.command.CommandFailedEvent;
import de.ozgcloud.command.CommandTestFactory;

class CollaborationEventListenerTest {

	@Spy
	@InjectMocks
	private CollaborationEventListener eventListener;

	@Mock
	private CurrentUserService currentUserService;
	@Mock
	private ApplicationEventPublisher eventPublisher;
	@Mock
	private CollaborationService collaborationService;
	@Mock
	private FachstelleService fachstelleService;
	@Mock
	private CollaborationRequestMapper commandMapper;

	@Nested
	class TestOnCreateCollaborationRequest {
		private final Command command = CommandTestFactory.create();

		@Test
		void shouldCallRunWithSecurityContext() {
			eventListener.onCreateCollaborationRequest(CommandCreatedEventTestFactory.withCommand(command));

			verify(eventListener).runWithSecurityContext(eq(command), any());
		}

		@Test
		void shouldExecuteCreateBescheid() {
			eventListener.onCreateCollaborationRequest(CommandCreatedEventTestFactory.withCommand(command));

			verify(eventListener).createCollaborationRequest(command);
		}
	}

	@Nested
	class TestCreateCollaborationRequest {

		private final Command command = CommandTestFactory.createBuilder().build();

		@Captor
		private ArgumentCaptor<CommandFailedEvent> commandFailedEventCaptor;

		@Test
		void shouldCallCommandMapper() {
			when(commandMapper.toCollaborationRequestExcludingFachstelle(any())).thenReturn(CollaborationRequestTestFactory.create());

			eventListener.createCollaborationRequest(command);

			verify(commandMapper).toCollaborationRequestExcludingFachstelle(command);
		}

		@Test
		void shouldCallCreateLevel1CollaborationRequest() {
			var collaborationRequest = CollaborationRequestTestFactory.create();
			when(commandMapper.toCollaborationRequestExcludingFachstelle(any())).thenReturn(collaborationRequest);

			eventListener.createCollaborationRequest(command);

			verify(eventListener).createLevel1CollaborationRequest(collaborationRequest, command);
		}

		@Test
		void shouldCallCreateLevel4CollaborationRequest() {
			var collaborationRequest = CollaborationRequestTestFactory.createBuilder().collaborationLevel(4).build();
			when(commandMapper.toCollaborationRequestExcludingFachstelle(any())).thenReturn(collaborationRequest);

			eventListener.createCollaborationRequest(command);

			verify(eventListener).createLevel4CollaborationRequest(collaborationRequest, command);
		}

		@DisplayName("should decline request")
		@ParameterizedTest(name = "when collaboration level is {0}")
		@ValueSource(ints = { 0, 2, 3, 5 })
		void shouldDeclineRequest(int collaborationLevel) {
			var collaborationRequest = CollaborationRequestTestFactory.createBuilder().collaborationLevel(collaborationLevel).build();
			when(commandMapper.toCollaborationRequestExcludingFachstelle(any())).thenReturn(collaborationRequest);

			eventListener.createCollaborationRequest(command);

			verify(eventPublisher).publishEvent(commandFailedEventCaptor.capture());
			assertThat(commandFailedEventCaptor.getValue().getSource()).isEqualTo(command.getId());
		}
	}

	@Nested
	class TestCreateLevel1CollaborationRequest {

		private final Map<String, Object> bodyObject = Map.of(
				CollaborationRequest.PROPERTY_TITEL, CollaborationRequestTestFactory.TITEL,
				CollaborationRequest.PROPERTY_BESCHREIBUNG, CollaborationRequestTestFactory.BESCHREIBUNG,
				CollaborationRequest.PROPERTY_ZUSTAENDIGE_STELLE, FachstelleTestFactory.TECHNICAL_ID,
				CollaborationRequest.PROPERTY_COLLABORATION_LEVEL, CollaborationRequestTestFactory.COLLABORATION_LEVEL);
		private final Command command = CommandTestFactory.createBuilder()
				.bodyObject(bodyObject)
				.build();
		private final CollaborationRequest collaborationRequest = CollaborationRequestTestFactory.createBuilder()
				.zustaendigeStelle(null)
				.build();

		@Captor
		private ArgumentCaptor<CollaborationRequest> collaborationRequestCaptor;

		@Test
		void shouldCallFachstelleService() {
			createLevel1CollaborationRequest();

			verify(fachstelleService).getOrganisationEinheit(FachstelleTestFactory.TECHNICAL_ID);
		}

		@Test
		void shouldCallCreateCollaborationRequest() {
			when(fachstelleService.getOrganisationEinheit(any())).thenReturn(CollaborationRequestTestFactory.ZUSTAENDIGE_STELLE);

			createLevel1CollaborationRequest();

			verify(collaborationService).createCollaborationRequest(collaborationRequestCaptor.capture());
			assertThat(collaborationRequestCaptor.getValue()).usingRecursiveComparison().isEqualTo(CollaborationRequestTestFactory.create());
		}

		private void createLevel1CollaborationRequest() {
			eventListener.createLevel1CollaborationRequest(collaborationRequest, command);
		}
	}

	@Nested
	class TestCreateLevel4CollaborationRequest {

		private final Map<String, Object> bodyObject = Map.of(
				CollaborationRequest.PROPERTY_TITEL, CollaborationRequestTestFactory.TITEL,
				CollaborationRequest.PROPERTY_BESCHREIBUNG, CollaborationRequestTestFactory.BESCHREIBUNG,
				CollaborationRequest.PROPERTY_ZUSTAENDIGE_STELLE, FachstelleTestFactory.TECHNICAL_ID,
				CollaborationRequest.PROPERTY_COLLABORATION_LEVEL, CollaborationRequestTestFactory.COLLABORATION_LEVEL);
		private final Command command = CommandTestFactory.createBuilder()
				.bodyObject(bodyObject)
				.build();
		private final CollaborationRequest collaborationRequest = CollaborationRequestTestFactory.createBuilder()
				.zustaendigeStelle(null)
				.build();

		@Captor
		private ArgumentCaptor<CollaborationRequest> collaborationRequestCaptor;

		@Test
		void shouldCallFachstelleService() {
			createLevel4CollaborationRequest();

			verify(fachstelleService).getExterneFachstelle(FachstelleTestFactory.TECHNICAL_ID);
		}

		@Test
		void shouldCallCreateFachstellenBeteiligungRequest() {
			when(fachstelleService.getExterneFachstelle(any())).thenReturn(CollaborationRequestTestFactory.ZUSTAENDIGE_STELLE);

			createLevel4CollaborationRequest();

			verify(collaborationService).createFachstellenBeteiligungRequest(collaborationRequestCaptor.capture());
			assertThat(collaborationRequestCaptor.getValue()).usingRecursiveComparison().isEqualTo(CollaborationRequestTestFactory.create());
		}

		private void createLevel4CollaborationRequest() {
			eventListener.createLevel4CollaborationRequest(collaborationRequest, command);
		}
	}

	@Nested
	class TestRunWithSecurityContext {

		private final Command command = CommandTestFactory.createBuilder().build();

		@Mock
		private Consumer<Command> commandExecutor;
		@Mock
		private SecurityContext secContext;
		@Captor
		private ArgumentCaptor<CommandFailedEvent> commandFailedEventCaptor;

		@BeforeEach
		void init() {
			when(currentUserService.startSecurityContext(any())).thenReturn(secContext);
		}

		@Test
		void shouldStartSecurityContext() {
			eventListener.runWithSecurityContext(command, commandExecutor);

			verify(currentUserService).startSecurityContext(command);
		}

		@Test
		void shouldExecuteCommand() {
			eventListener.runWithSecurityContext(command, commandExecutor);

			verify(commandExecutor).accept(command);
		}

		@Test
		void shouldResetSecurityContext() {
			eventListener.runWithSecurityContext(command, commandExecutor);

			verify(currentUserService).resetSecurityContext(secContext);
		}

		@Test
		void shouldResetSecurityContextAfterException() {
			doThrow(new RuntimeException("ups")).when(commandExecutor).accept(any());

			eventListener.runWithSecurityContext(command, commandExecutor);

			verify(currentUserService).resetSecurityContext(secContext);
		}

		@Test
		void shouldPublishCommandFailedEvent() {
			doThrow(new RuntimeException("ups")).when(commandExecutor).accept(any());

			eventListener.runWithSecurityContext(command, commandExecutor);

			verify(eventPublisher).publishEvent(commandFailedEventCaptor.capture());
			assertThat(commandFailedEventCaptor.getValue().getSource()).isEqualTo(command.getId());
			assertThat(commandFailedEventCaptor.getValue().getErrorMessage()).isNotEmpty();
		}
	}
}