/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.OutputStream;
import java.util.UUID;
import java.util.function.Consumer;
import java.util.function.Function;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.core.task.TaskExecutor;
import org.springframework.test.util.ReflectionTestUtils;

import com.google.protobuf.ByteString;

import de.ozgcloud.collaboration.vorgang.CollaborationVorgangMapper;
import de.ozgcloud.collaboration.vorgang.GrpcVorgangTestFactory;
import de.ozgcloud.collaboration.vorgang.Vorgang;
import de.ozgcloud.collaboration.vorgang.VorgangService;
import de.ozgcloud.collaboration.vorgang.VorgangTestFactory;
import de.ozgcloud.command.CommandTestFactory;
import de.ozgcloud.common.binaryfile.GrpcBinaryFileServerDownloader;
import io.grpc.stub.CallStreamObserver;
import io.grpc.stub.StreamObserver;

class CollaborationGrpcServiceTest {

	@Spy
	@InjectMocks
	private CollaborationGrpcService grpcService;
	@Mock
	private CollaborationService service;
	@Mock
	private CollaborationVorgangMapper vorgangMapper;

	@Mock
	private VorgangService vorgangService;

	@Mock
	private TaskExecutor taskExecutor;

	@Nested
	class TestFindVorgang {

		private static final Vorgang VORGANG = VorgangTestFactory.create();

		@Mock
		private GrpcFindVorgangResponse grpcResponse;
		@Mock
		private StreamObserver<GrpcFindVorgangResponse> responseObserver;

		@BeforeEach
		void init() {
			doReturn(grpcResponse).when(grpcService).buildFindVorgangResponse(any());
			when(vorgangService.getVorgang(anyString())).thenReturn(VORGANG);
		}

		@Test
		void shouldCallGetVorgang() {
			findVorgang();

			verify(vorgangService).getVorgang(CommandTestFactory.VORGANG_ID);
		}

		@Test
		void shouldCallBuildFindVorgangResponse() {
			findVorgang();

			verify(grpcService).buildFindVorgangResponse(VORGANG);
		}

		@Test
		void shouldCallOnNext() {
			findVorgang();

			verify(responseObserver).onNext(grpcResponse);
		}

		@Test
		void shouldCallOnCompleted() {
			findVorgang();

			verify(responseObserver).onCompleted();
		}

		private void findVorgang() {
			grpcService.findVorgang(GrpcFindVorgangRequestTestFactory.create(), responseObserver);
		}
	}

	@Nested
	class TestBuildFindVorgangResponse {

		private final Vorgang vorgang = VorgangTestFactory.create();
		private final GrpcVorgang grpcVorgang = GrpcVorgangTestFactory.create();

		@BeforeEach
		void init() {
			when(vorgangMapper.toGrpc(any())).thenReturn(GrpcVorgangTestFactory.create());
		}

		@Test
		void shouldCallMapper() {
			buildFindVorgangResponse();

			verify(vorgangMapper).toGrpc(vorgang);
		}

		@Test
		void shouldSetVorgang() {
			var response = buildFindVorgangResponse();

			assertThat(response.getVorgang()).isEqualTo(grpcVorgang);
		}

		private GrpcFindVorgangResponse buildFindVorgangResponse() {
			return grpcService.buildFindVorgangResponse(vorgang);
		}
	}

	@DisplayName("Get file content")
	@Nested
	class TestGetFileContent {

		@Mock
		private StreamObserver<GrpcGetFileContentResponse> responseObserver;
		@Mock
		private GrpcBinaryFileServerDownloader<GrpcGetFileContentResponse> grpcDownloader;

		private final String fileId = UUID.randomUUID().toString();
		private final GrpcGetFileContentRequest request = GrpcGetFileContentRequest.newBuilder().setId(fileId).build();

		@BeforeEach
		void mock() {
			doReturn(grpcDownloader).when(grpcService).buildDownloader(any(), any());
		}

		@Test
		void shouldCreateDownloader() {
			grpcService.getFileContent(request, responseObserver);

			verify(grpcService).buildDownloader(fileId, responseObserver);
		}

		@Test
		void shouldStartDownload() {
			grpcService.getFileContent(request, responseObserver);

			verify(grpcDownloader).start();
		}
	}

	@DisplayName("build downloader")
	@Nested
	class TestBuildDownloader {

		private final String fileId = UUID.randomUUID().toString();

		@Mock
		private CallStreamObserver<GrpcGetFileContentResponse> callStreamObserver;

		@Test
		void shouldContainCallObserver() {
			var downloader = grpcService.buildDownloader(fileId, callStreamObserver);

			assertThat(getCallObserver(downloader)).isEqualTo(callStreamObserver);
		}

		@Test
		void shouldContainDownloadConsumer() {
			var downloader = grpcService.buildDownloader(fileId, callStreamObserver);

			assertThat(getDownloadConsumer(downloader)).isNotNull();
		}

		@Test
		void shouldContainChunkBuilder() {
			var downloader = grpcService.buildDownloader(fileId, callStreamObserver);

			assertThat(getChunkBuilder(downloader)).isNotNull();
		}

		@Test
		void shouldContainTaskBuilder() {
			var downloader = grpcService.buildDownloader(fileId, callStreamObserver);

			assertThat(getTaskExecuter(downloader)).isEqualTo(taskExecutor);
		}

		@SuppressWarnings("unchecked")
		private CallStreamObserver<GrpcGetFileContentResponse> getCallObserver(
				GrpcBinaryFileServerDownloader<GrpcGetFileContentResponse> downloader) {
			return (CallStreamObserver<GrpcGetFileContentResponse>) ReflectionTestUtils.getField(downloader, "callObserver");
		}

		@SuppressWarnings("unchecked")
		private Consumer<OutputStream> getDownloadConsumer(GrpcBinaryFileServerDownloader<GrpcGetFileContentResponse> downloader) {
			return (Consumer<OutputStream>) ReflectionTestUtils.getField(downloader, "downloadConsumer");
		}

		@SuppressWarnings("unchecked")
		private Function<ByteString, GrpcGetFileContentResponse> getChunkBuilder(
				GrpcBinaryFileServerDownloader<GrpcGetFileContentResponse> downloader) {
			return (Function<ByteString, GrpcGetFileContentResponse>) ReflectionTestUtils.getField(downloader, "chunkBuilder");
		}

		private TaskExecutor getTaskExecuter(GrpcBinaryFileServerDownloader<GrpcGetFileContentResponse> downloader) {
			return (TaskExecutor) ReflectionTestUtils.getField(downloader, "taskExecutor");
		}
	}
}