/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Map;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.apilib.common.command.OzgCloudCommand;
import de.ozgcloud.apilib.common.command.grpc.CommandMapper;
import de.ozgcloud.apilib.common.datatypes.GenericId;
import de.ozgcloud.apilib.user.OzgCloudUserId;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangId;
import de.ozgcloud.collaboration.common.callcontext.CollaborationManagerCallContextGrpcClientInterceptor;
import de.ozgcloud.collaboration.common.vorgang.attached_item.VorgangAttachedItem;
import de.ozgcloud.collaboration.common.vorgang.attached_item.VorgangAttachedItemTestFactory;
import de.ozgcloud.collaboration.fachstelle.FachstelleMapper;
import de.ozgcloud.collaboration.fachstelle.FachstelleService;
import de.ozgcloud.collaboration.fachstelle.FachstelleTestFactory;
import de.ozgcloud.collaboration.request.GrpcCreateCollaborationRequestDataTestFactory;
import de.ozgcloud.collaboration.vorgang.GrpcCollaborationRequestTestFactory;
import de.ozgcloud.collaboration.vorgang.VorgangTestFactory;
import de.ozgcloud.command.Command;
import de.ozgcloud.command.CommandTestFactory;
import de.ozgcloud.common.datatype.StringBasedValue;

class CollaborationRequestMapperTest {

	@Spy
	@InjectMocks
	private CollaborationRequestMapper mapper = Mappers.getMapper(CollaborationRequestMapper.class);

	@Mock
	private CommandMapper commandMapper;
	@Mock
	private FachstelleService fachstelleService;
	@Mock
	private FachstelleMapper fachstelleMapper;

	@Nested
	class TestToCollaborationRequestExcludingFachstelle {

		private static final Map<String, Object> BODY_OBJECT = Map.of(
				CollaborationRequest.PROPERTY_TITEL, CollaborationRequestTestFactory.TITEL,
				CollaborationRequest.PROPERTY_BESCHREIBUNG, CollaborationRequestTestFactory.BESCHREIBUNG,
				CollaborationRequest.PROPERTY_ZUSTAENDIGE_STELLE, FachstelleTestFactory.TECHNICAL_ID,
				CollaborationRequest.PROPERTY_COLLABORATION_LEVEL, CollaborationRequestTestFactory.COLLABORATION_LEVEL);
		private static final Command COMMAND = CommandTestFactory.createBuilder().bodyObject(
				BODY_OBJECT)
				.vorgangId(VorgangTestFactory.ID_STR)
				.build();

		@Test
		void shouldMapCommand() {
			var result = mapper.toCollaborationRequestExcludingFachstelle(COMMAND);

			assertThat(result).usingRecursiveComparison()
					.ignoringFields("id", "createdBy", "collaborationVorgangId", "zustaendigeStelle")
					.isEqualTo(CollaborationRequestTestFactory.create());
		}
	}

	@Nested
	class TestToOzgCloudCommand {

		private static final CollaborationRequest COLLABORATION_REQUEST = CollaborationRequestTestFactory.create();
		private static final OzgCloudVorgangId OZG_CLOUD_VORGANG_ID = OzgCloudVorgangId.from(CommandTestFactory.VORGANG_ID);
		private static final StringBasedValue OZG_CLOUD_RELATION_ID = GenericId.from(CommandTestFactory.RELATION_ID);
		private static final OzgCloudUserId OZG_CLOUD_USER_ID = OzgCloudUserId.from(CommandTestFactory.CREATED_BY);
		private static final Map<String, Object> BODY_OBJECT = Map.of("key", "value");

		@BeforeEach
		void init() {
			when(commandMapper.toOzgCloudVorgangId(any())).thenReturn(OZG_CLOUD_VORGANG_ID);
			when(commandMapper.mapRelationId(any())).thenReturn(OZG_CLOUD_RELATION_ID);
			when(commandMapper.toOzgCloudUserId(any())).thenReturn(OZG_CLOUD_USER_ID);
			doReturn(BODY_OBJECT).when(mapper).toBodyObject(any());
		}

		@Test
		void shouldNotSetId() {
			var result = toOzgCloudCommand();

			assertThat(result.getId()).isNull();
		}

		@Test
		void shouldCallOzgCloudVorgangIdMapper() {
			toOzgCloudCommand();

			verify(commandMapper).toOzgCloudVorgangId(VorgangTestFactory.ID_STR);
		}

		@Test
		void shouldSetVorgangId() {
			var command = toOzgCloudCommand();

			assertThat(command.getVorgangId()).isEqualTo(OZG_CLOUD_VORGANG_ID);
		}

		@Test
		void shouldCallMapRelationId() {
			toOzgCloudCommand();

			verify(commandMapper).mapRelationId(VorgangTestFactory.ID_STR);
		}

		@Test
		void shouldSetRelationId() {
			var command = toOzgCloudCommand();

			assertThat(command.getRelationId()).isEqualTo(OZG_CLOUD_RELATION_ID);
		}

		@Test
		void shouldCallMapOzgCloudUser() {
			toOzgCloudCommand();

			verify(commandMapper).toOzgCloudUserId(CommandTestFactory.CREATED_BY);
		}

		@Test
		void shouldSetCreatedBy() {
			var command = toOzgCloudCommand();

			assertThat(command.getCreatedBy()).isEqualTo(OZG_CLOUD_USER_ID);
		}

		@Test
		void shouldSetOrder() {
			var command = toOzgCloudCommand();

			assertThat(command.getOrder()).isEqualTo(CollaborationService.CREATE_ATTACHED_ITEM_ORDER);
		}

		@Test
		void shouldSetBodyObject() {
			var command = toOzgCloudCommand();

			assertThat(command.getBodyObject()).isEqualTo(BODY_OBJECT);
		}

		private OzgCloudCommand toOzgCloudCommand() {
			return mapper.toOzgCloudCommand(COLLABORATION_REQUEST, CollaborationService.CREATE_ATTACHED_ITEM_ORDER);
		}
	}

	@Nested
	class TestToBodyObject {

		private static final CollaborationRequest REQUEST = CollaborationRequestTestFactory.create();

		@Test
		void shouldSetVorgangId() {
			var result = mapper.toBodyObject(CollaborationRequestTestFactory.create());

			assertThat(result).containsEntry(CollaborationRequestMapper.FIELD_ITEM_VORGANG_ID, VorgangTestFactory.ID_STR);
		}

		@Test
		void shouldSetClient() {
			var result = mapper.toBodyObject(CollaborationRequestTestFactory.create());

			assertThat(result).containsEntry(CollaborationRequestMapper.FIELD_ITEM_CLIENT,
					CollaborationManagerCallContextGrpcClientInterceptor.COLLABORATION_MANAGER_CLIENT_NAME);
		}

		@Test
		void shouldSetItemName() {
			var result = mapper.toBodyObject(CollaborationRequestTestFactory.create());

			assertThat(result).containsEntry(CollaborationRequestMapper.FIELD_ITEM_NAME,
					CollaborationRequestMapper.ITEM_NAME);
		}

		@Test
		void shouldCallToItemName() {
			toBodyObject();

			verify(mapper).toItemMap(any());
		}

		@Test
		void shouldSetItemMap() {
			var itemMap = Map.<String, Object>of("key", "value");
			doReturn(itemMap).when(mapper).toItemMap(any());

			var result = toBodyObject();

			assertThat(result).containsEntry(CollaborationRequestMapper.FIELD_ITEM_BODY, itemMap);
		}

		private Map<String, Object> toBodyObject() {
			return mapper.toBodyObject(REQUEST);
		}
	}

	@Nested
	class TestToItemMap {

		@Test
		void shouldSetTitel() {
			var result = toItemMap();

			assertThat(result).containsEntry(CollaborationRequest.PROPERTY_TITEL, CollaborationRequestTestFactory.TITEL);
		}

		@Test
		void shouldSetBeschreibung() {
			var result = toItemMap();

			assertThat(result).containsEntry(CollaborationRequest.PROPERTY_BESCHREIBUNG, CollaborationRequestTestFactory.BESCHREIBUNG);
		}

		@Test
		void shouldCallFachstelleMapper() {
			toItemMap();

			verify(fachstelleMapper).toMap(CollaborationRequestTestFactory.ZUSTAENDIGE_STELLE);
		}

		@Test
		void shouldSetZustaendigeStelle() {
			var fachstelleMap = FachstelleTestFactory.createAsMap();
			when(fachstelleMapper.toMap(CollaborationRequestTestFactory.ZUSTAENDIGE_STELLE)).thenReturn(fachstelleMap);

			var result = toItemMap();

			assertThat(result).containsEntry(CollaborationRequest.PROPERTY_ZUSTAENDIGE_STELLE, fachstelleMap);
		}

		@Test
		void shouldSetCollaborationVorgangId() {
			var result = toItemMap();

			assertThat(result).containsEntry(CollaborationRequest.PROPERTY_COLLABORATION_VORGANG_ID,
					CollaborationRequestTestFactory.COLLABORATION_VORGANG_ID);
		}

		@Test
		void shouldSetCollaborationLevel() {
			var result = toItemMap();

			assertThat(result).containsEntry(CollaborationRequest.PROPERTY_COLLABORATION_LEVEL, CollaborationRequestTestFactory.COLLABORATION_LEVEL);
		}

		private Map<String, Object> toItemMap() {
			return mapper.toItemMap(CollaborationRequestTestFactory.create());
		}
	}

	@DisplayName("Map from vorgangAttachedItem")
	@Nested
	class TestMapFromVorgangAttachedItem {
		private static final Map<String, Object> COLLABORATION_REQUEST_MAP = CollaborationRequestTestFactory.createAsMap();
		private static final VorgangAttachedItem VORGANG_ATTACHED_ITEM = VorgangAttachedItemTestFactory.createBuilder()
				.itemName("CollaborationRequest")
				.item(COLLABORATION_REQUEST_MAP)
				.build();

		@Test
		void shouldCallGetZustaendigeStelleFromItemMap() {
			mapper.mapFromVorgangAttachedItem(VORGANG_ATTACHED_ITEM);

			verify(mapper).getZustaendigeStelleFromItemMap(COLLABORATION_REQUEST_MAP);
		}

		@Test
		void shouldMap() {
			doReturn(FachstelleTestFactory.create()).when(mapper).getZustaendigeStelleFromItemMap(any());

			var item = mapper.mapFromVorgangAttachedItem(VORGANG_ATTACHED_ITEM);

			assertThat(item).usingRecursiveComparison().isEqualTo(CollaborationRequestTestFactory.create());
		}
	}

	@Nested
	class TestGetZustaendigeStelleFromItemMap {

		private final Map<String, Object> itemMap = CollaborationRequestTestFactory.createAsMap();

		@Test
		void shouldMapFachstelle() {
			mapper.getZustaendigeStelleFromItemMap(itemMap);

			verify(fachstelleMapper).fromMap(CollaborationRequestTestFactory.FACHSTELLE_MAP);
		}

		@Test
		void shouldReturnFachstelle() {
			var expectedFachstelle = FachstelleTestFactory.create();
			when(fachstelleMapper.fromMap(any())).thenReturn(expectedFachstelle);

			var actualFachstelle = mapper.getZustaendigeStelleFromItemMap(itemMap);

			assertThat(actualFachstelle).isSameAs(expectedFachstelle);
		}
	}

	@Nested
	class TestToGrpcCollaborationRequest {

		@BeforeEach
		void callCommandMapper() {
			when(commandMapper.valueToString(any())).thenCallRealMethod();
		}

		@Test
		void shouldMap() {
			var request = mapper.toGrpcCollaborationRequest(CollaborationRequestTestFactory.create());

			assertThat(request).isEqualTo(GrpcCollaborationRequestTestFactory.create());
		}
	}

	@Nested
	class TestToGrpcCreateCollaborationRequestData {

		@Test
		void shouldMap() {
			var expectedRequestData = GrpcCreateCollaborationRequestDataTestFactory.createBuilder()
					.setVorgangId(VorgangTestFactory.ID_STR)
					.build();

			var requestData = mapper.toGrpcCreateCollaborationRequestData(CollaborationRequestTestFactory.create());

			assertThat(requestData).isEqualTo(expectedRequestData);
		}
	}
}