/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration;

import static org.junit.jupiter.api.Assertions.*;

import jakarta.validation.ConstraintViolationException;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.NullAndEmptySource;
import org.junit.jupiter.params.provider.ValueSource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.annotation.DirtiesContext;

import de.ozgcloud.collaboration.fachstelle.FachstelleTestFactory;
import de.ozgcloud.common.test.ITCase;

@ITCase
@DirtiesContext
class CollaborationServiceITCase {

	@Autowired
	private CollaborationService service;

	@Nested
	class TestCreateCollaborationRequest {

		@DisplayName("should validate titel")
		@ParameterizedTest(name = "fail when titel: \"{0}\"")
		@NullAndEmptySource
		@ValueSource(strings = { StringUtils.SPACE })
		void shouldValidateTitel(String titel) {
			var request = CollaborationRequestTestFactory.createBuilder().titel(titel).build();

			assertThrows(ConstraintViolationException.class, () -> service.createCollaborationRequest(request));
		}

		@DisplayName("should validate beschreibung")
		@ParameterizedTest(name = "fail when beschreibung: \"{0}\"")
		@NullAndEmptySource
		@ValueSource(strings = { StringUtils.SPACE })
		void shouldValidateBeschreibung(String beschreibung) {
			var request = CollaborationRequestTestFactory.createBuilder().beschreibung(beschreibung).build();

			assertThrows(ConstraintViolationException.class, () -> service.createCollaborationRequest(request));
		}

		@DisplayName("should validate technicalId of zustaendigeStelle")
		@ParameterizedTest(name = "fail when technicalId: \"{0}\"")
		@NullAndEmptySource
		@ValueSource(strings = { StringUtils.SPACE })
		void shouldValidateCollaborationZustaendigeStelleTechnicalId(String technicalId) {
			var request = CollaborationRequestTestFactory.createBuilder()
					.zustaendigeStelle(FachstelleTestFactory.createBuilder().technicalId(technicalId).build())
					.build();

			assertThrows(ConstraintViolationException.class, () -> service.createCollaborationRequest(request));
		}

		@DisplayName("should validate subjectId of zustaendigeStelle")
		@ParameterizedTest(name = "fail when subjectId: \"{0}\"")
		@NullAndEmptySource
		@ValueSource(strings = { StringUtils.SPACE })
		void shouldValidateCollaborationZustaendigeStelleSubjectId(String subjectId) {
			var request = CollaborationRequestTestFactory.createBuilder()
					.zustaendigeStelle(FachstelleTestFactory.createBuilder().subjectId(subjectId).build())
					.build();

			assertThrows(ConstraintViolationException.class, () -> service.createCollaborationRequest(request));
		}

		@DisplayName("should validate type of zustaendigeStelle")
		@Test
		void shouldValidateCollaborationZustaendigeStelleType() {
			var request = CollaborationRequestTestFactory.createBuilder()
					.zustaendigeStelle(FachstelleTestFactory.createBuilder().type(null).build())
					.build();

			assertThrows(ConstraintViolationException.class, () -> service.createCollaborationRequest(request));
		}

		@DisplayName("should validate zustaendigeStelle")
		@Test
		void shouldValidateCollaborationZustaendigeStelle() {
			var request = CollaborationRequestTestFactory.createBuilder()
					.zustaendigeStelle(null)
					.build();

			assertThrows(ConstraintViolationException.class, () -> service.createCollaborationRequest(request));
		}
	}

	@Nested
	class TestCreateFachstellenBeteiligungRequest {

		@DisplayName("validate titel")
		@ParameterizedTest(name = "should throw exception on titel: \"{0}\"")
		@NullAndEmptySource
		@ValueSource(strings = { StringUtils.SPACE })
		void shouldValidateTitel(String titel) {
			var request = CollaborationRequestTestFactory.createBuilder().titel(titel).build();

			assertThrows(ConstraintViolationException.class, () -> service.createFachstellenBeteiligungRequest(request));
		}

		@DisplayName("should validate technicalId of zustaendigeStelle")
		@ParameterizedTest(name = "fail when technicalId: \"{0}\"")
		@NullAndEmptySource
		@ValueSource(strings = { StringUtils.SPACE })
		void shouldValidateCollaborationZustaendigeStelleTechnicalId(String technicalId) {
			var request = CollaborationRequestTestFactory.createBuilder()
					.zustaendigeStelle(FachstelleTestFactory.createBuilder().technicalId(technicalId).build())
					.build();

			assertThrows(ConstraintViolationException.class, () -> service.createFachstellenBeteiligungRequest(request));
		}

		@DisplayName("should validate zustaendigeStelle")
		@ParameterizedTest(name = "fail when subjectId: \"{0}\"")
		@NullAndEmptySource
		@ValueSource(strings = { StringUtils.SPACE })
		void shouldValidateCollaborationZustaendigeStelleSubjectId(String subjectId) {
			var request = CollaborationRequestTestFactory.createBuilder()
					.zustaendigeStelle(FachstelleTestFactory.createBuilder().subjectId(subjectId).build())
					.build();

			assertThrows(ConstraintViolationException.class, () -> service.createFachstellenBeteiligungRequest(request));
		}
	}
}