/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.assertj.core.api.InstanceOfAssertFactories.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.apilib.common.command.OzgCloudCommand;
import de.ozgcloud.apilib.common.command.OzgCloudCommandService;
import de.ozgcloud.apilib.common.command.OzgCloudCreateSubCommandsRequest;
import de.ozgcloud.apilib.common.command.OzgCloudCreateSubCommandsRequestTestFactory;
import de.ozgcloud.apilib.common.command.grpc.CommandMapper;
import de.ozgcloud.apilib.common.datatypes.GenericId;
import de.ozgcloud.apilib.user.OzgCloudUserId;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangId;
import de.ozgcloud.collaboration.common.freemarker.TemplateHandler;
import de.ozgcloud.collaboration.common.user.UserId;
import de.ozgcloud.collaboration.common.user.UserProfile;
import de.ozgcloud.collaboration.common.user.UserProfileService;
import de.ozgcloud.collaboration.fachstelle.FachstelleTestFactory;
import de.ozgcloud.collaboration.vorgang.ServiceKonto;
import de.ozgcloud.collaboration.vorgang.Vorgang;
import de.ozgcloud.collaboration.vorgang.VorgangService;
import de.ozgcloud.collaboration.vorgang.VorgangTestFactory;
import de.ozgcloud.command.CommandTestFactory;
import de.ozgcloud.common.datatype.StringBasedValue;
import de.ozgcloud.common.errorhandling.TechnicalException;

class CollaborationServiceTest {

	@Spy
	@InjectMocks
	private CollaborationService service;

	@Mock
	private VorgangService vorgangService;
	@Mock
	private OzgCloudCommandService commandService;
	@Mock
	private UserProfileService userProfileService;
	@Mock
	private CollaborationRequestMapper collaborationRequestMapper;
	@Mock
	private CommandMapper commandMapper;
	@Mock
	private TemplateHandler templateHandler;
	@Mock
	private FachstelleVorgangUrlProvider provider;
	@Mock
	private Optional<FachstelleVorgangUrlProvider> urlProvider;

	@Nested
	class TestCreateCollaborationRequest {

		private static final CollaborationRequest COLLABORATION_REQUEST = CollaborationRequestTestFactory.createBuilder().collaborationVorgangId(null)
				.createdBy(null).build();
		private static final CollaborationRequest ENRICHED_COLLABORATION_REQUEST = CollaborationRequestTestFactory.create();

		@Mock
		private OzgCloudCreateSubCommandsRequest createSubCommandsRequest;

		@BeforeEach
		void init() {
			when(vorgangService.createCollaborationVorgang(any())).thenReturn(CollaborationRequestTestFactory.COLLABORATION_VORGANG_ID);
			doReturn(createSubCommandsRequest).when(service).buildCreateCollaborationRequestRequest(any());
			doReturn(ENRICHED_COLLABORATION_REQUEST).when(service).enrichCollaborationRequest(any(), any());
		}

		@Test
		void shouldCallCreateCollaborationVorgang() {
			createCollaborationRequest();

			verify(vorgangService).createCollaborationVorgang(COLLABORATION_REQUEST);
		}

		@Test
		void shouldCallEnrichCollaborationRequest() {
			createCollaborationRequest();

			verify(service).enrichCollaborationRequest(COLLABORATION_REQUEST, CollaborationRequestTestFactory.COLLABORATION_VORGANG_ID);
		}

		@Test
		void shouldCallBuildCreateSubCommandsRequest() {
			createCollaborationRequest();

			verify(service).buildCreateCollaborationRequestRequest(ENRICHED_COLLABORATION_REQUEST);
		}

		@Test
		void shouldCallAddSubCommands() {
			createCollaborationRequest();

			verify(commandService).addSubCommands(createSubCommandsRequest);
		}

		private void createCollaborationRequest() {
			service.createCollaborationRequest(COLLABORATION_REQUEST);
		}
	}

	@Nested
	class TestEnrichCollaborationRequest {

		@BeforeEach
		void init() {
			doReturn(CommandTestFactory.CREATED_BY).when(service).getUserId();
		}

		@Test
		void shouldSetCollaborationVorgangId() {
			var result = enrichCollaborationRequest();

			assertThat(result.getCollaborationVorgangId()).isEqualTo(CollaborationRequestTestFactory.COLLABORATION_VORGANG_ID);
		}

		@Test
		void shouldCallGetUserId() {
			enrichCollaborationRequest();

			verify(service).getUserId();
		}

		@Test
		void shouldSetCreatedBy() {
			var result = enrichCollaborationRequest();

			assertThat(result.getCreatedBy()).isEqualTo(CommandTestFactory.CREATED_BY);
		}

		private CollaborationRequest enrichCollaborationRequest() {
			return service.enrichCollaborationRequest(CollaborationRequestTestFactory.create(),
					CollaborationRequestTestFactory.COLLABORATION_VORGANG_ID);
		}
	}

	@Nested
	class TestGetUserId {

		private static final UserId USER_ID = UserId.from(CommandTestFactory.CREATED_BY);

		@Mock
		private UserProfile userProfile;

		@BeforeEach
		void init() {
			when(userProfile.getId()).thenReturn(USER_ID);
			when(userProfileService.getUserProfile()).thenReturn(userProfile);
		}

		@Test
		void shouldCallGetUserProfile() {
			service.getUserId();

			verify(userProfileService).getUserProfile();
		}

		@Test
		void shouldReturnUserId() {
			var result = service.getUserId();

			assertThat(result).isEqualTo(CommandTestFactory.CREATED_BY);
		}
	}

	@Nested
	class TestBuildCreateCollaborationRequestRequest {

		private static final CollaborationRequest COLLABORATION_REQUEST = CollaborationRequestTestFactory.createBuilder().collaborationVorgangId(null)
				.createdBy(null).build();

		@Mock
		private OzgCloudCommand subCommand;

		@BeforeEach
		void init() {
			when(collaborationRequestMapper.toOzgCloudCommand(any(), any())).thenReturn(subCommand);
		}

		@Test
		void shouldSetParentId() {
			var result = buildCreateCollaborationRequestRequest();

			assertThat(result.getParentId()).isEqualTo(CommandTestFactory.ID);
		}

		@Test
		void shouldSetExecutionMode() {
			var result = buildCreateCollaborationRequestRequest();

			assertThat(result.getExecutionMode()).isEqualTo(CollaborationService.SUBCOMMANDS_EXECUTION_MODE);
		}

		@Test
		void shouldSetCompletedIfSubsCompleted() {
			var result = buildCreateCollaborationRequestRequest();

			assertThat(result.isCompletedIfSubsCompleted()).isTrue();
		}

		@Test
		void shouldCallToOzgCloudCommand() {
			buildCreateCollaborationRequestRequest();

			verify(collaborationRequestMapper).toOzgCloudCommand(COLLABORATION_REQUEST, CollaborationService.CREATE_ATTACHED_ITEM_ORDER);
		}

		@Test
		void shouldSetSubCommands() {
			var result = buildCreateCollaborationRequestRequest();

			assertThat(result.getSubCommands()).containsExactly(subCommand);
		}

		private OzgCloudCreateSubCommandsRequest buildCreateCollaborationRequestRequest() {
			return service.buildCreateCollaborationRequestRequest(COLLABORATION_REQUEST);
		}
	}

	@Nested
	class TestCreateFachstellenBeteiligungRequest {

		@Nested
		class TestCreateRequest {
			private static final Vorgang VORGANG = VorgangTestFactory.create();
			private static final CollaborationRequest ENRICHED_REQUEST = CollaborationRequestTestFactory.create();

			@Mock
			private OzgCloudCommand subCommand;

			@BeforeEach
			void init() {
				doReturn(List.of(subCommand)).when(service).buildSubCommands(any());
				when(vorgangService.getVorgang(any())).thenReturn(VORGANG);
				doReturn(ENRICHED_REQUEST).when(service).enrichCollaborationRequest(any(), any());
			}

			@Test
			void shouldCallGetVorgang() {
				createFachstellenBeteiligungRequest();

				verify(vorgangService).getVorgang(VorgangTestFactory.ID_STR);
			}

			@Test
			void shouldCallEnrichCollaborationRequest() {
				var collaborationRequest = CollaborationRequestTestFactory.create();

				service.createFachstellenBeteiligungRequest(collaborationRequest);

				verify(service).enrichCollaborationRequest(collaborationRequest, VorgangTestFactory.ID.toString());
			}

			@Test
			void shouldCallBuildSubCommands() {
				createFachstellenBeteiligungRequest();

				verify(service).buildSubCommands(ENRICHED_REQUEST);
			}

			@Test
			void shouldCallBuildCreateSubCommandsRequest() {
				createFachstellenBeteiligungRequest();

				verify(service).buildCreateSubCommandsRequest(ENRICHED_REQUEST, List.of(subCommand));
			}

			@Test
			void shouldCallAddSubCommands() {
				var addSubCommandsRequest = OzgCloudCreateSubCommandsRequestTestFactory.create();
				doReturn(addSubCommandsRequest).when(service).buildCreateSubCommandsRequest(any(), any());

				createFachstellenBeteiligungRequest();

				verify(commandService).addSubCommands(addSubCommandsRequest);
			}
		}

		@Test
		void shouldThrowExceptionIfFachstelleNotConfigured() {
			when(urlProvider.isEmpty()).thenReturn(true);

			assertThrows(TechnicalException.class, this::createFachstellenBeteiligungRequest);
		}

		private void createFachstellenBeteiligungRequest() {
			service.createFachstellenBeteiligungRequest(CollaborationRequestTestFactory.create());
		}
	}

	@Nested
	class TestBuildCreateSubCommandsRequest {

		@Mock
		private OzgCloudCommand subCommand;

		@Test
		void shouldSetParentId() {
			var result = buildCreateSubCommandsRequest();

			assertThat(result.getParentId()).isEqualTo(CommandTestFactory.ID);
		}

		@Test
		void shouldSetExecutionMode() {
			var result = buildCreateSubCommandsRequest();

			assertThat(result.getExecutionMode()).isEqualTo(CollaborationService.SUBCOMMANDS_EXECUTION_MODE);
		}

		@Test
		void shouldSetCompletedIfSubsCompleted() {
			var result = buildCreateSubCommandsRequest();

			assertThat(result.isCompletedIfSubsCompleted()).isTrue();
		}

		@Test
		void shouldSetSubCommands() {
			var result = buildCreateSubCommandsRequest();

			assertThat(result.getSubCommands()).containsExactly(subCommand);
		}

		private OzgCloudCreateSubCommandsRequest buildCreateSubCommandsRequest() {
			return service.buildCreateSubCommandsRequest(CollaborationRequestTestFactory.create(), List.of(subCommand));
		}
	}

	@Nested
	class TestBuildSubCommands {

		private static final CollaborationRequest COLLABORATION_REQUEST = CollaborationRequestTestFactory.create();

		@Mock
		private OzgCloudCommand persistCollaborationRequestCommand;
		@Mock
		private OzgCloudCommand sendPostfachNachrichtCommand;

		@BeforeEach
		void init() {
			when(collaborationRequestMapper.toOzgCloudCommand(any(), any())).thenReturn(persistCollaborationRequestCommand);
			doReturn(sendPostfachNachrichtCommand).when(service).buildSendPostfachNachrichtCommand(any());
		}

		@Test
		void shouldCallCollaborationRequestMapper() {
			buildSubCommands();

			verify(collaborationRequestMapper).toOzgCloudCommand(any(), eq(CollaborationService.CREATE_ATTACHED_ITEM_ORDER));
		}

		@Test
		void shouldCallBuildSendPostfachNachrichtCommand() {
			buildSubCommands();

			verify(service).buildSendPostfachNachrichtCommand(COLLABORATION_REQUEST);
		}

		@Test
		void shouldAddSubCommands() {
			var result = buildSubCommands();

			assertThat(result).containsExactly(persistCollaborationRequestCommand, sendPostfachNachrichtCommand);
		}

		private List<OzgCloudCommand> buildSubCommands() {
			return service.buildSubCommands(COLLABORATION_REQUEST);
		}
	}

	@Nested
	class TestBuildSendPostfachNachrichtCommand {

		private static final CollaborationRequest COLLABORATION_REQUEST = CollaborationRequestTestFactory.create();
		private static final OzgCloudUserId OZG_CLOUD_ID = OzgCloudUserId.from(CommandTestFactory.CREATED_BY);
		private static final Map<String, Object> OBJECT_MAP = Map.of("key", "value");
		private static final OzgCloudVorgangId VORGANG_ID = OzgCloudVorgangId.from(VorgangTestFactory.ID.toString());
		private static final StringBasedValue RELATION_ID = GenericId.from(VorgangTestFactory.ID.toString());

		@BeforeEach
		void init() {
			when(commandMapper.toOzgCloudVorgangId(anyString())).thenReturn(VORGANG_ID);
			when(commandMapper.mapRelationId(anyString())).thenReturn(RELATION_ID);
			when(commandMapper.toOzgCloudUserId(anyString())).thenReturn(OZG_CLOUD_ID);
			doReturn(OBJECT_MAP).when(service).buildPostfachSendNachrichtCommandBody(any(), any());
		}

		@Test
		void shouldCallBuildPostfachAddress() {
			buildSendPostfachNachrichtCommand();

			verify(service).buildMukPostfachAddress(FachstelleTestFactory.SUBJECT_ID);
		}

		@Test
		void shouldCallBuildSenNachrichtCommandBody() {
			var postfachAddressMap = Map.<String, Object>of("key", "value");
			doReturn(postfachAddressMap).when(service).buildMukPostfachAddress(any());

			buildSendPostfachNachrichtCommand();

			verify(service).buildPostfachSendNachrichtCommandBody(postfachAddressMap, VorgangTestFactory.ID_STR);
		}

		@Test
		void shouldSetBodyObject() {
			var result = buildSendPostfachNachrichtCommand();

			assertThat(result.getBodyObject()).isSameAs(OBJECT_MAP);
		}

		@Test
		void shouldCallMapVorgangId() {
			buildSendPostfachNachrichtCommand();

			verify(commandMapper).toOzgCloudVorgangId(VorgangTestFactory.ID_STR);
		}

		@Test
		void shouldSetVorgangId() {
			var result = buildSendPostfachNachrichtCommand();

			assertThat(result.getVorgangId()).isEqualTo(VORGANG_ID);
		}

		@Test
		void shouldCallMapRelationId() {
			buildSendPostfachNachrichtCommand();

			verify(commandMapper).mapRelationId(VorgangTestFactory.ID_STR);
		}

		@Test
		void shouldSetRelationId() {
			var result = buildSendPostfachNachrichtCommand();

			assertThat(result.getRelationId()).isEqualTo(RELATION_ID);
		}

		@Test
		void shouldSetOrder() {
			var result = buildSendPostfachNachrichtCommand();

			assertThat(result.getOrder()).isEqualTo(CollaborationService.SEND_POSTFACH_NACHRICHT_ORDER);
		}

		@Test
		void shouldCallMapToOzgCloudUserId() {
			buildSendPostfachNachrichtCommand();

			verify(commandMapper).toOzgCloudUserId(CommandTestFactory.CREATED_BY);
		}

		@Test
		void shouldSetCreatedBy() {
			var result = buildSendPostfachNachrichtCommand();

			assertThat(result.getCreatedBy()).isEqualTo(OZG_CLOUD_ID);
		}

		private OzgCloudCommand buildSendPostfachNachrichtCommand() {
			return service.buildSendPostfachNachrichtCommand(COLLABORATION_REQUEST);
		}
	}

	@Nested
	class TestBuildPostfachAddress {

		@Test
		void shouldSetType() {
			var result = buildPostfachAddress();

			assertThat(result).containsEntry(ServiceKonto.PostfachAddress.FIELD_TYPE, CollaborationService.MUK_POSTFACH_ADDRESS_TYPE);
		}

		@Test
		void shouldSetVersion() {
			var result = buildPostfachAddress();

			assertThat(result).containsEntry(ServiceKonto.PostfachAddress.FIELD_VERSION, CollaborationService.MUK_POSTFACH_ADDRESS_VERSION);
		}

		@Test
		void shouldSetIdentifier() {
			var result = buildPostfachAddress();

			assertThat(result).extracting(ServiceKonto.PostfachAddress.FIELD_IDENTIFIER, MAP)
					.containsEntry(ServiceKonto.PostfachAddress.FIELD_IDENTIFIER_POSTFACH_ID, FachstelleTestFactory.SUBJECT_ID);
		}

		@Test
		void shouldSetServicekontoType() {
			var result = buildPostfachAddress();

			assertThat(result).containsEntry(ServiceKonto.FIELD_SERVICEKONTO_TYPE, CollaborationService.MUK_SERVICEKONTO_TYPE);
		}

		private Map<String, Object> buildPostfachAddress() {
			return service.buildMukPostfachAddress(FachstelleTestFactory.SUBJECT_ID);
		}
	}

	@Nested
	class TestBuildSendNachrichtCommandBody {

		private static final Map<String, Object> POSTFACH_ADDRESS = Map.of("key", "value");

		@BeforeEach
		void init() {
			doReturn(CollaborationRequestTestFactory.BESCHREIBUNG).when(service).getAnfrageText(any());
		}

		@Test
		void shouldSetReplyOption() {
			var result = buildSendNachrichtCommandBody();

			assertThat(result).containsEntry(CollaborationService.FIELD_REPLY_OPTION, CollaborationService.REPLY_OPTION);
		}

		@Test
		void shouldSetSubject() {
			var result = buildSendNachrichtCommandBody();

			assertThat(result).containsEntry(CollaborationService.FIELD_SUBJECT, CollaborationService.SUBJECT);
		}

		@Test
		void shouldSetMailBody() {
			var result = buildSendNachrichtCommandBody();

			assertThat(result).containsEntry(CollaborationService.FIELD_MAIL_BODY, CollaborationRequestTestFactory.BESCHREIBUNG);
		}

		private Map<String, Object> buildSendNachrichtCommandBody() {
			return service.buildPostfachSendNachrichtCommandBody(POSTFACH_ADDRESS, VorgangTestFactory.ID_STR);
		}
	}

	@Nested
	class TestGetAnfrageText {

		private static final String DEFAULT_MAIL_TEXT = "default mail text";

		private static final Map<String, Object> TEMPLATE_PARAMETERS = Map.of("key", "value ");

		@BeforeEach
		void init() {
			when(templateHandler.fillTemplate(anyString(), any())).thenReturn(DEFAULT_MAIL_TEXT);
			doReturn(TEMPLATE_PARAMETERS).when(service).getTemplateParameters(any());
		}

		@Test
		void shouldCallGetTemplateParameters() {
			getAnfrageText();

			verify(service).getTemplateParameters(VorgangTestFactory.ID_STR);
		}

		@Test
		void shouldCallTemplateHandler() {
			getAnfrageText();

			verify(templateHandler).fillTemplate(CollaborationService.COLLABORATION_NACHRICHT_TEMPLATE, TEMPLATE_PARAMETERS);
		}

		@Test
		void shouldReturnDefaultMailText() {
			var result = getAnfrageText();

			assertThat(result).isEqualTo(DEFAULT_MAIL_TEXT);
		}

		private String getAnfrageText() {
			return service.getAnfrageText(VorgangTestFactory.ID_STR);
		}
	}

	@Nested
	class TestGetTemplateParameters {

		private final String linkToVorgang = LoremIpsum.getInstance().getUrl();

		@BeforeEach
		void init() {
			when(urlProvider.get()).thenReturn(provider);
		}

		@Test
		void shouldCallUrlProvider() {
			when(provider.getUrl(any())).thenReturn(linkToVorgang);

			getTemplateParameters();

			verify(provider).getUrl(VorgangTestFactory.ID_STR);
		}

		@Test
		void shouldReturnTemplateParameters() {
			when(provider.getUrl(any())).thenReturn(linkToVorgang);

			var result = getTemplateParameters();

			assertThat(result).containsExactly(Map.entry(CollaborationService.KEY_VORGANG_LINK, linkToVorgang));
		}

		private Map<String, Object> getTemplateParameters() {
			return service.getTemplateParameters(VorgangTestFactory.ID_STR);
		}
	}
}