/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.Mock;
import org.springframework.test.util.ReflectionTestUtils;

import com.thedeanda.lorem.LoremIpsum;

class FachstelleVorgangUrlProviderTest {

	private static final String FACHSTELLE_HOST = "https://" + LoremIpsum.getInstance().getWords(1);
	private static final String VORGANG_MANAGER_SERVICE_ADDRESS = "vorgang-manager.test-namespace";

	private FachstelleVorgangUrlProvider provider;

	@Mock
	private FachstelleProperties fachstelleProperties;
	@Mock
	private VorgangManagerProperties vorgangManagerProperties;

	@BeforeEach
	void init() {
		when(fachstelleProperties.getHost()).thenReturn(FACHSTELLE_HOST);
		when(vorgangManagerProperties.getServiceAddress()).thenReturn(VORGANG_MANAGER_SERVICE_ADDRESS);
		provider = spy(new FachstelleVorgangUrlProvider(fachstelleProperties, vorgangManagerProperties));
	}

	@Nested
	class TestSetVorgangUrlPrefix {

		@Test
		void shouldBuildVorgangUrlPrefix() {
			var encodedResourceUri = FachstelleVorgangUrlProvider.QUERY_PARAM_URI + VORGANG_MANAGER_SERVICE_ADDRESS + "%2Fvorgangs%2F";
			var vorgagUrlPrefix = FACHSTELLE_HOST + FachstelleVorgangUrlProvider.URL_PATH_RESOURCES + encodedResourceUri;

			var provider = createProvider();

			assertThat(provider).extracting("vorgangUrlPrefix").isEqualTo(vorgagUrlPrefix);
		}

		private FachstelleVorgangUrlProvider createProvider() {
			return spy(new FachstelleVorgangUrlProvider(fachstelleProperties, vorgangManagerProperties));
		}
	}

	@Nested
	class TestGetFachstelleHost {

		@ParameterizedTest
		@ValueSource(strings = { "http://localhost", "https://localhost" })
		void shouldReturnUnchanged(String fachstelleHost) {
			when(fachstelleProperties.getHost()).thenReturn(fachstelleHost);

			var result = provider.getFachstelleHost(fachstelleProperties);

			assertThat(result).isEqualTo(fachstelleHost);
		}

		@Test
		void shouldAddHttps() {
			when(fachstelleProperties.getHost()).thenReturn("localhost");

			var result = provider.getFachstelleHost(fachstelleProperties);

			assertThat(result).isEqualTo("https://localhost");
		}
	}

	@Nested
	class TestExtractServiceDomainName {

		@ParameterizedTest
		@ValueSource(strings = {
				"dns:///vorgang-manager.by-kiel-dev:9090",
				"dns:///vorgang-manager.by-kiel-dev",
				"vorgang-manager.by-kiel-dev:9090",
				"vorgang-manager.by-kiel-dev",
				"dns://vorgang-manager.by-kiel-dev:9090",
				"dns:/vorgang-manager.by-kiel-dev:9090",
				"dns:/vorgang-manager.by-kiel-dev:",
				"dns:/vorgang-manager.by-kiel-dev:1",
				"abc:/vorgang-manager.by-kiel-dev:9090" })
		void shouldReturnDomainName(String serviceAddress) {
			when(vorgangManagerProperties.getServiceAddress()).thenReturn(serviceAddress);

			var address = provider.getServiceName(vorgangManagerProperties);

			assertThat(address).isEqualTo("vorgang-manager.by-kiel-dev");
		}
	}

	@Nested
	class TestGetUrl {

		private static final String VORGANG_URL_PREFIX = LoremIpsum.getInstance().getUrl();

		@Test
		void shouldReturnUrl() {
			ReflectionTestUtils.setField(provider, "vorgangUrlPrefix", VORGANG_URL_PREFIX);
			var vorgangId = UUID.randomUUID().toString();

			var url = provider.getUrl(vorgangId);

			assertThat(url).isEqualTo(VORGANG_URL_PREFIX + vorgangId);
		}
	}

}
