/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration.common.user;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.Mock;

import de.ozgcloud.apilib.user.OzgCloudUserId;
import de.ozgcloud.apilib.user.OzgCloudUserProfile;
import de.ozgcloud.apilib.user.OzgCloudUserProfileService;
import de.ozgcloud.collaboration.common.callcontext.CallContextUser;
import de.ozgcloud.collaboration.common.callcontext.CurrentUserService;
import de.ozgcloud.command.CommandTestFactory;
import de.ozgcloud.common.errorhandling.TechnicalException;

class UserProfileServiceTest {

	private UserProfileService service;

	@Mock
	private CurrentUserService currentUserService;
	@Mock
	private OzgCloudUserProfileService ozgCloudUserProfileService;
	@Mock
	private CollaborationUserProfileMapper mapper;

	@Nested
	class TestGetUserProfile {

		@Mock
		private CallContextUser contextUser;
		@Mock
		private OzgCloudUserProfile ozgCloudUserProfile;
		@Mock
		private UserProfile userProfile;

		@Captor
		private ArgumentCaptor<OzgCloudUserId> userIdCaptor;

		@BeforeEach
		void init() {
			service = new UserProfileService(currentUserService, Optional.of(ozgCloudUserProfileService), mapper);

			when(contextUser.getUserId()).thenReturn(Optional.of(CommandTestFactory.CREATED_BY));
			when(currentUserService.getUser()).thenReturn(contextUser);
			when(ozgCloudUserProfileService.getById(any())).thenReturn(ozgCloudUserProfile);
			when(mapper.mapProfile(any())).thenReturn(userProfile);
		}

		@Test
		void shouldCallGetById() {
			when(ozgCloudUserProfileService.getById(any())).thenReturn(ozgCloudUserProfile);

			service.getUserProfile();

			verify(ozgCloudUserProfileService).getById(userIdCaptor.capture());
			assertThat(userIdCaptor.getValue()).hasToString(CommandTestFactory.CREATED_BY);
		}

		@Test
		void shouldCallMapProfile() {
			when(ozgCloudUserProfileService.getById(any())).thenReturn(ozgCloudUserProfile);

			service.getUserProfile();

			verify(mapper).mapProfile(ozgCloudUserProfile);
		}

		@Test
		void shouldReturnUserProfile() {
			var result = service.getUserProfile();

			assertThat(result).isEqualTo(userProfile);
		}
	}

	@Nested
	class TestGetUserProfileErrors {

		@Mock
		private CallContextUser contextUser;

		@Test
		void shouldThrowExceptionWhenOzgCloudServiceMissing() {
			service = new UserProfileService(currentUserService, Optional.empty(), mapper);

			Assertions.assertThrows(IllegalStateException.class, service::getUserProfile);
		}

		@Test
		void shouldThrowExceptionWhenUserIdMissing() {
			when(currentUserService.getUser()).thenReturn(contextUser);

			service = new UserProfileService(currentUserService, Optional.of(ozgCloudUserProfileService), mapper);

			Assertions.assertThrows(TechnicalException.class, service::getUserProfile);
		}
	}
}