/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration.common.vorgang.attached_item;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.collaboration.vorgang.VorgangTestFactory;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcFindVorgangAttachedItemRequest;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcFindVorgangAttachedItemResponse;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcVorgangAttachedItemRequest;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcVorgangAttachedItemResponse;
import de.ozgcloud.vorgang.vorgangAttachedItem.VorgangAttachedItemServiceGrpc.VorgangAttachedItemServiceBlockingStub;
import io.grpc.ClientInterceptor;

class VorgangAttachedItemRemoteServiceTest {

	@Spy
	@InjectMocks
	private VorgangAttachedItemRemoteService service;
	@Mock
	private VorgangAttachedItemServiceBlockingStub serviceStub;
	@Mock
	private CollaborationManagerVorgangAttachedItemMapper mapper;

	@Mock
	private ClientInterceptor clientInterceptor;

	@DisplayName("Get by id")
	@Nested
	class TestGetById {

		@Captor
		private ArgumentCaptor<GrpcVorgangAttachedItemRequest> requestCaptor;

		private final GrpcVorgangAttachedItemResponse response = GrpcVorgangAttachedItemResponseTestFactory.create();
		private final VorgangAttachedItem vorgangAttachedItem = VorgangAttachedItemTestFactory.create();

		@BeforeEach
		void mock() {
			when(service.getStub()).thenReturn(serviceStub);
			when(serviceStub.getById(any())).thenReturn(response);
			when(mapper.fromGrpc(any())).thenReturn(vorgangAttachedItem);
		}

		@Test
		void shouldGetServiceStub() {
			getById();

			verify(service).getStub();
		}

		@Test
		void shouldCallServiceStub() {
			getById();

			verify(serviceStub).getById(requestCaptor.capture());
			assertThat(requestCaptor.getValue().getId()).isEqualTo(VorgangAttachedItemTestFactory.ID);
		}

		@Test
		void shouldCallMapper() {
			getById();

			verify(mapper).fromGrpc(GrpcVorgangAttachedItemResponseTestFactory.VORGANG_ATTACHED_ITEM);
		}

		@Test
		void shouldReturnValue() {
			var item = getById();

			assertThat(item).usingRecursiveComparison().isEqualTo(vorgangAttachedItem);
		}

		private VorgangAttachedItem getById() {
			return service.getById(VorgangAttachedItemTestFactory.ID);
		}
	}

	@DisplayName("Find by vorgangId")
	@Nested
	class TestFindByVorgangId {

		@Captor
		private ArgumentCaptor<GrpcFindVorgangAttachedItemRequest> requestCaptor;

		private final GrpcFindVorgangAttachedItemResponse response = GrpcFindVorgangAttachedItemResponseTestFactory.create();
		private final VorgangAttachedItem vorgangAttachedItem = VorgangAttachedItemTestFactory.create();

		@BeforeEach
		void mock() {
			when(service.getStub()).thenReturn(serviceStub);
			when(serviceStub.find(any())).thenReturn(response);
			when(mapper.fromGrpc(any())).thenReturn(vorgangAttachedItem);
		}

		@Test
		void shouldGetServiceStub() {
			findByVorgangId();

			verify(service).getStub();
		}

		@Test
		void shouldCallServiceStub() {
			findByVorgangId();

			verify(serviceStub).find(requestCaptor.capture());
			assertThat(requestCaptor.getValue().getVorgangId()).isEqualTo(VorgangTestFactory.ID_STR);
			assertThat(requestCaptor.getValue().getItemName()).isEqualTo(VorgangAttachedItemTestFactory.ITEM_NAME);
		}

		@Test
		void shouldCallMapper() {
			findByVorgangId();

			verify(mapper).fromGrpc(GrpcFindVorgangAttachedItemResponseTestFactory.VORGANG_ATTACHED_ITEM);
		}

		@Test
		void shouldReturnValue() {
			var items = findByVorgangId();

			assertThat(items).hasSize(1);
			assertThat(items.getFirst()).isEqualTo(vorgangAttachedItem);
		}

		private List<VorgangAttachedItem> findByVorgangId() {
			return service.findByVorgangId(VorgangTestFactory.ID_STR, VorgangAttachedItemTestFactory.ITEM_NAME).toList();
		}
	}

	@DisplayName("Get stub")
	@Nested
	class TestGetStub {

		@BeforeEach
		void mock() {
			when(serviceStub.withInterceptors(any())).thenReturn(serviceStub);
		}

		@Test
		void shouldAddInterceptorToStub() {
			service.getStub();

			verify(serviceStub).withInterceptors(clientInterceptor);
		}

		@Test
		void shouldReturnStub() {
			var stub = service.getStub();

			assertThat(stub).isEqualTo(serviceStub);
		}
	}
}