/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration.fachstelle;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.collaboration.common.callcontext.CollaborationManagerCallContextGrpcClientInterceptor;
import de.ozgcloud.zufi.grpc.fachstelle.FachstelleServiceGrpc.FachstelleServiceBlockingStub;
import de.ozgcloud.zufi.grpc.organisationseinheit.OrganisationsEinheitServiceGrpc.OrganisationsEinheitServiceBlockingStub;

class FachstelleRemoteServiceTest {

	@Spy
	@InjectMocks
	private FachstelleRemoteService service;

	@Mock
	private FachstelleServiceBlockingStub fachstelleServiceBlockingStub;
	@Mock
	private OrganisationsEinheitServiceBlockingStub organisationsEinheitServiceBlockingStub;
	@Mock
	private FachstelleMapper fachstelleMapper;
	@Mock
	private CollaborationManagerCallContextGrpcClientInterceptor callContextInterceptor;

	@Nested
	class TestGetExterneFachstelle {

		@Mock
		private FachstelleServiceBlockingStub fachstelleServiceBlockingStubWithInterceptor;

		@BeforeEach
		void mock() {
			doReturn(fachstelleServiceBlockingStubWithInterceptor).when(service).getFachstelleServiceBlockingStub();
			when(fachstelleServiceBlockingStubWithInterceptor.getById(any())).thenReturn(GrpcFachstelleGetResponseTestFactory.create());
		}

		@Test
		void shouldCallGetFachstelleServiceBlockingStub() {
			getExterneFachstelle();

			verify(service).getFachstelleServiceBlockingStub();
		}

		@Test
		void shouldCallFachstelleServiceBlockingStub() {
			getExterneFachstelle();

			verify(fachstelleServiceBlockingStubWithInterceptor).getById(GrpcFachstelleGetRequestTestFactory.create());
		}

		@Test
		void shouldCallFachstelleMapper() {
			getExterneFachstelle();

			verify(fachstelleMapper).fromExterneFachstelle(GrpcFachstelleGetResponseTestFactory.EXTERNE_FACHSTELLE);
		}

		@Test
		void shouldReturnFachstelle() {
			var expectedFachstelle = FachstelleTestFactory.create();
			when(fachstelleMapper.fromExterneFachstelle(any())).thenReturn(expectedFachstelle);

			var returnFachstelle = getExterneFachstelle();

			assertThat(returnFachstelle).isSameAs(expectedFachstelle);
		}

		private Fachstelle getExterneFachstelle() {
			return service.getExterneFachstelle(FachstelleTestFactory.TECHNICAL_ID);
		}
	}

	@Nested
	class TestGetFachstelleServiceBlockingStub {

		@Mock
		private FachstelleServiceBlockingStub fachstelleServiceBlockingStubWithInterceptor;

		@Test
		void shouldAttachInterceptor() {
			service.getFachstelleServiceBlockingStub();

			verify(fachstelleServiceBlockingStub).withInterceptors(callContextInterceptor);
		}

		@Test
		void shouldReturnStubWithInterceptor() {
			when(fachstelleServiceBlockingStub.withInterceptors(any())).thenReturn(fachstelleServiceBlockingStubWithInterceptor);

			var stub = service.getFachstelleServiceBlockingStub();

			assertThat(stub).isSameAs(fachstelleServiceBlockingStubWithInterceptor);
		}
	}

	@Nested
	class TestGetOrganisationEinheit {

		@Mock
		private OrganisationsEinheitServiceBlockingStub organisationsEinheitServiceBlockingStubWithInterceptor;

		@BeforeEach
		void mock() {
			doReturn(organisationsEinheitServiceBlockingStubWithInterceptor).when(service).getOrganisationsEinheitServiceBlockingStub();
			when(organisationsEinheitServiceBlockingStubWithInterceptor.getById(any()))
					.thenReturn(GrpcOrganisationsEinheitGetResponseTestFactory.create());
		}

		@Test
		void shouldCallGetOrganisationsEinheitServiceBlockingStub() {
			getOrganisationEinheit();

			verify(service).getOrganisationsEinheitServiceBlockingStub();
		}

		@Test
		void shouldCallOrganisationsEinheitServiceBlockingStub() {
			getOrganisationEinheit();

			verify(organisationsEinheitServiceBlockingStubWithInterceptor).getById(GrpcOrganisationsEinheitGetRequestTestFactory.create());
		}

		@Test
		void shouldCallFachstelleMapper() {
			getOrganisationEinheit();

			verify(fachstelleMapper).fromOrganisationEinheit(GrpcOrganisationsEinheitGetResponseTestFactory.ORGANISATION_EINHEIT);
		}

		@Test
		void shouldReturnFachstelle() {
			var expectedFachstelle = FachstelleTestFactory.create();
			when(fachstelleMapper.fromOrganisationEinheit(any())).thenReturn(expectedFachstelle);

			var returnFachstelle = getOrganisationEinheit();

			assertThat(returnFachstelle).isSameAs(expectedFachstelle);
		}

		private Fachstelle getOrganisationEinheit() {
			return service.getOrganisationEinheit(FachstelleTestFactory.TECHNICAL_ID);
		}
	}

	@Nested
	class TestGetOrganisationsEinheitServiceBlockingStub {

		@Mock
		private OrganisationsEinheitServiceBlockingStub organisationsEinheitServiceBlockingStubWithInterceptor;

		@Test
		void shouldAttachInterceptor() {
			service.getOrganisationsEinheitServiceBlockingStub();

			verify(organisationsEinheitServiceBlockingStub).withInterceptors(callContextInterceptor);
		}

		@Test
		void shouldReturnStubWithInterceptor() {
			when(organisationsEinheitServiceBlockingStub.withInterceptors(any())).thenReturn(organisationsEinheitServiceBlockingStubWithInterceptor);

			var stub = service.getOrganisationsEinheitServiceBlockingStub();

			assertThat(stub).isSameAs(organisationsEinheitServiceBlockingStubWithInterceptor);
		}
	}
}
