/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration.request;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.collaboration.CollaborationRequest;
import de.ozgcloud.collaboration.CollaborationRequestMapper;
import de.ozgcloud.collaboration.CollaborationRequestTestFactory;
import de.ozgcloud.collaboration.vorgang.GrpcCollaborationRequestTestFactory;
import de.ozgcloud.collaboration.vorgang.VorgangTestFactory;
import io.grpc.stub.StreamObserver;

class CollaborationRequestGrpcServiceTest {

	@InjectMocks
	private CollaborationRequestGrpcService grpcService;
	@Mock
	private CollaborationRequestService service;
	@Mock
	private CollaborationRequestMapper mapper = Mappers.getMapper(CollaborationRequestMapper.class);

	@DisplayName("Get request")
	@Nested
	class TestGetRequest {

		@Mock
		private StreamObserver<GrpcGetRequestResponse> streamObserver;
		@Captor
		private ArgumentCaptor<GrpcGetRequestResponse> responseCaptor;

		private final GrpcGetRequestRequest grpcRequest = GrpcGetRequestRequestTestFactory.create();
		private final CollaborationRequest collaborationRequest = CollaborationRequestTestFactory.create();
		private final GrpcCollaborationRequest grpcCollaborationRequest = GrpcCollaborationRequestTestFactory.create();

		@BeforeEach
		void mock() {
			when(service.getById(any())).thenReturn(collaborationRequest);
			when(mapper.toGrpcCollaborationRequest(any())).thenReturn(grpcCollaborationRequest);
		}

		@Test
		void shouldCallService() {
			getRequest();

			verify(service).getById(CollaborationRequestTestFactory.ID_STR);
		}

		@Test
		void shouldCallMapper() {
			getRequest();

			verify(mapper).toGrpcCollaborationRequest(collaborationRequest);
		}

		@Test
		void shouldCallOnNext() {
			var responseRequest = getRequest();

			assertThat(responseRequest).usingRecursiveComparison().isEqualTo(GrpcCollaborationRequestTestFactory.create());
		}

		@Test
		void shouldCallOnCompleted() {
			getRequest();

			verify(streamObserver).onCompleted();
		}

		private GrpcCollaborationRequest getRequest() {
			grpcService.getRequest(grpcRequest, streamObserver);
			verify(streamObserver).onNext(responseCaptor.capture());
			return responseCaptor.getValue().getRequest();
		}
	}

	@DisplayName("Find requests")
	@Nested
	class TestFindRequests {

		@Mock
		private StreamObserver<GrpcFindRequestsResponse> streamObserver;
		@Captor
		private ArgumentCaptor<GrpcFindRequestsResponse> responseCaptor;

		private final GrpcFindRequestsRequest grpcRequest = GrpcFindRequestsRequestTestFactory.create();
		private final CollaborationRequest collaborationRequest = CollaborationRequestTestFactory.create();
		private final GrpcCollaborationRequest grpcCollaborationRequest = GrpcCollaborationRequestTestFactory.create();

		@BeforeEach
		void mock() {
			when(service.findByVorgangId(any())).thenReturn(Stream.of(collaborationRequest));
			when(mapper.toGrpcCollaborationRequest(any())).thenReturn(grpcCollaborationRequest);
		}

		@Test
		void shouldCallService() {
			findRequests();

			verify(service).findByVorgangId(VorgangTestFactory.ID_STR);
		}

		@Test
		void shouldCallMapper() {
			findRequests();

			verify(mapper).toGrpcCollaborationRequest(collaborationRequest);
		}

		@Test
		void shouldCallOnNext() {
			findRequests();

			verify(streamObserver).onNext(responseCaptor.capture());
			assertThat(responseCaptor.getValue().getRequestsList().getFirst()).isEqualTo(grpcCollaborationRequest);
		}

		@Test
		void shouldCallOnCompleted() {
			findRequests();

			verify(streamObserver).onCompleted();
		}

		private void findRequests() {
			grpcService.findRequests(grpcRequest, streamObserver);
		}
	}
}
