/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration.request;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.collaboration.CollaborationRequest;
import de.ozgcloud.collaboration.CollaborationRequestMapper;
import de.ozgcloud.collaboration.CollaborationRequestTestFactory;
import de.ozgcloud.collaboration.common.vorgang.attached_item.VorgangAttachedItem;
import de.ozgcloud.collaboration.common.vorgang.attached_item.VorgangAttachedItemService;
import de.ozgcloud.collaboration.common.vorgang.attached_item.VorgangAttachedItemTestFactory;
import de.ozgcloud.collaboration.vorgang.VorgangTestFactory;

class CollaborationRequestServiceTest {

	@InjectMocks
	private CollaborationRequestService service;
	@Mock
	private VorgangAttachedItemService vorgangAttachedItemService;
	@Mock
	private CollaborationRequestMapper mapper;

	@DisplayName("Get by id")
	@Nested
	class TestGetById {

		private final VorgangAttachedItem vorgangAttachedItem = VorgangAttachedItemTestFactory.create();
		private final CollaborationRequest collaborationRequest = CollaborationRequestTestFactory.create();

		@BeforeEach
		void mock() {
			when(vorgangAttachedItemService.getById(any())).thenReturn(vorgangAttachedItem);
			when(mapper.mapFromVorgangAttachedItem(any())).thenReturn(collaborationRequest);
		}

		@Test
		void shouldCallVorgangAttachedItemService() {
			getById();

			verify(vorgangAttachedItemService).getById(CollaborationRequestTestFactory.ID_STR);
		}

		@Test
		void shouldCallMapper() {
			getById();

			verify(mapper).mapFromVorgangAttachedItem(vorgangAttachedItem);
		}

		@Test
		void shouldReturnRequest() {
			var request = getById();

			assertThat(request).isSameAs(collaborationRequest);
		}

		private CollaborationRequest getById() {
			return service.getById(CollaborationRequestTestFactory.ID_STR);
		}
	}

	@DisplayName("Find by vorgangId")
	@Nested
	class TestFindByVorgangId {

		private final VorgangAttachedItem vorgangAttachedItem = VorgangAttachedItemTestFactory.create();
		private final CollaborationRequest collaborationRequest = CollaborationRequestTestFactory.create();

		@BeforeEach
		void mock() {
			when(vorgangAttachedItemService.findByVorgangId(any(), any())).thenReturn(Stream.of(vorgangAttachedItem));
			when(mapper.mapFromVorgangAttachedItem(any())).thenReturn(collaborationRequest);
		}

		@Test
		void shouldCallVorgangAttachedItemService() {
			findByVorgangId();

			verify(vorgangAttachedItemService).findByVorgangId(VorgangTestFactory.ID_STR, CollaborationRequestService.VORGANG_ATTACHED_ITEM_NAME);
		}

		@Test
		void shouldCallMapper() {
			findByVorgangId();

			verify(mapper).mapFromVorgangAttachedItem(vorgangAttachedItem);
		}

		@Test
		void shouldReturnRequestList() {
			var items = findByVorgangId();

			assertThat(items).hasSize(1);
			assertThat(items.get(0)).usingRecursiveComparison().isEqualTo(collaborationRequest);
		}

		private List<CollaborationRequest> findByVorgangId() {
			return service.findByVorgangId(VorgangTestFactory.ID_STR).toList();
		}
	}
}