/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.apilib.file.OzgCloudFile;
import de.ozgcloud.apilib.file.OzgCloudFileGroup;
import de.ozgcloud.apilib.file.OzgCloudFileTestFactory;
import de.ozgcloud.apilib.vorgang.OzgCloudAntragsteller;
import de.ozgcloud.apilib.vorgang.OzgCloudAntragstellerTestFactory;
import de.ozgcloud.apilib.vorgang.OzgCloudEingang;
import de.ozgcloud.apilib.vorgang.OzgCloudFormField;
import de.ozgcloud.apilib.vorgang.OzgCloudSubForm;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangEingangTestFactory;
import de.ozgcloud.apilib.vorgang.grpc.OzgCloudFileGroupTestFactory;
import de.ozgcloud.collaboration.common.file.CollaborationFileGroupMapper;
import de.ozgcloud.collaboration.common.file.CollaborationFileMapper;
import de.ozgcloud.collaboration.common.file.FileGroupTestFactory;
import de.ozgcloud.collaboration.common.file.FileTestFactory;
import de.ozgcloud.collaboration.common.file.GrpcFileGroupTestFactory;
import de.ozgcloud.collaboration.common.file.GrpcFileTestFactory;

class CollaborationEingangMapperTest {

	@InjectMocks
	private final CollaborationEingangMapper mapper = Mappers.getMapper(CollaborationEingangMapper.class);
	@Mock
	private final CollaborationAntragstellerMapper antragstellerMapper = Mappers.getMapper(CollaborationAntragstellerMapper.class);
	@Mock
	private final CollaborationFileGroupMapper fileGroupMapper = Mappers.getMapper(CollaborationFileGroupMapper.class);
	@Mock
	private final CollaborationFileMapper fileMapper = Mappers.getMapper(CollaborationFileMapper.class);
	@Mock
	private final CollaborationFormDataMapper formDataMapper = Mappers.getMapper(CollaborationFormDataMapper.class);

	@DisplayName("From ozgCloudEingang")
	@Nested
	class TestFromOzgCloudEingang {

		private final OzgCloudAntragsteller ozgCloudAntragsteller = OzgCloudAntragstellerTestFactory.create();
		private final OzgCloudFile ozgCloudFile = OzgCloudFileTestFactory.create();
		private final OzgCloudFileGroup ozgCloudFileGroup = OzgCloudFileGroupTestFactory.createBuilder()
				.clearFiles().file(ozgCloudFile)
				.build();
		private final OzgCloudFormField ozgCloudFormField = OzgCloudFormField.builder()
				.name(FormDataEntryTestFactory.FORM_FIELD_NAME)
				.label(FormDataEntryTestFactory.FORM_FIELD_LABEL)
				.value(FormDataEntryTestFactory.FORM_FIELD_VALUE)
				.build();

		private final OzgCloudFormField ozgCloudSubFormFormField = OzgCloudFormField.builder()
				.name(FormDataEntryTestFactory.SUB_FORM_FIELD_NAME)
				.label(FormDataEntryTestFactory.SUB_FORM_FIELD_LABEL)
				.value(FormDataEntryTestFactory.SUB_FORM_FIELD_VALUE)
				.build();

		private final OzgCloudSubForm ozgCloudSubForm = OzgCloudSubForm.builder()
				.name(FormDataEntryTestFactory.SUB_FORM_NAME)
				.label(FormDataEntryTestFactory.SUB_FORM_LABEL)
				.formEntry(ozgCloudSubFormFormField)
				.build();

		private final OzgCloudEingang ozgCloudEingang = OzgCloudVorgangEingangTestFactory.createBuilder()
				.antragsteller(ozgCloudAntragsteller)
				.clearAttachments().attachment(ozgCloudFileGroup)
				.clearRepresentations().representation(ozgCloudFile)
				.formDataEntrys(List.of(ozgCloudSubForm, ozgCloudFormField))
				.build();

		@BeforeEach
		void mock() {
			when(antragstellerMapper.fromOzgCloudAntragsteller(any())).thenReturn(AntragstellerTestFactory.create());
			when(fileGroupMapper.fromOzgCloudFileGroup(any())).thenReturn(FileGroupTestFactory.create());
			when(fileMapper.fromOzgCloudFile(any())).thenReturn(FileTestFactory.create());
			when(formDataMapper.fromOzgCloudEntries(any()))
					.thenReturn(List.of(FormDataEntryTestFactory.createFormField(), FormDataEntryTestFactory.createSubForm()));
		}

		@Test
		void shouldCallAntragstellerMapper() {
			mapper.fromOzgCloudEingang(ozgCloudEingang);

			verify(antragstellerMapper).fromOzgCloudAntragsteller(ozgCloudAntragsteller);
		}

		@Test
		void shouldCallFileGroupMapper() {
			mapper.fromOzgCloudEingang(ozgCloudEingang);

			verify(fileGroupMapper).fromOzgCloudFileGroup(ozgCloudFileGroup);
		}

		@Test
		void shouldCallFileMapper() {
			mapper.fromOzgCloudEingang(ozgCloudEingang);

			verify(fileMapper).fromOzgCloudFile(ozgCloudFile);
		}

		@Test
		void shouldMap() {
			var eingang = mapper.fromOzgCloudEingang(OzgCloudVorgangEingangTestFactory.create());

			assertThat(eingang).usingRecursiveComparison().isEqualTo(EingangTestFactory.create());
		}
	}

	@DisplayName("To grpc")
	@Nested
	class TestToGrpc {

		@BeforeEach
		void mock() {
			when(antragstellerMapper.toGrpc(any())).thenReturn(GrpcAntragstellerTestFactory.create());
			when(fileGroupMapper.toGrpc(any())).thenReturn(GrpcFileGroupTestFactory.create());
			when(fileMapper.toGrpc(any())).thenReturn(GrpcFileTestFactory.create());
			when(formDataMapper.toGrpcFormData(any())).thenReturn(GrpcFormDataTestFactory.create());
		}

		@Test
		void shouldCallAntragstellerMapper() {
			mapper.toGrpc(EingangTestFactory.create());

			verify(antragstellerMapper).toGrpc(EingangTestFactory.ANTRAGSTELLER);
		}

		@Test
		void shouldCallFileGroupMapper() {
			mapper.toGrpc(EingangTestFactory.create());

			verify(fileGroupMapper).toGrpc(EingangTestFactory.ATTACHMENT);
		}

		@Test
		void shouldCallFileMapper() {
			mapper.toGrpc(EingangTestFactory.create());

			verify(fileMapper).toGrpc(EingangTestFactory.REPRESENTATION);
		}

		@Test
		void shouldMap() {
			var eingang = mapper.toGrpc(EingangTestFactory.create());

			assertThat(eingang).usingRecursiveComparison().ignoringFields("memoizedHashCode").isEqualTo(GrpcEingangTestFactory.create());
		}
	}
}