/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration.vorgang;

import static de.ozgcloud.collaboration.vorgang.FormDataEntryTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.Spy;

import de.ozgcloud.apilib.vorgang.OzgCloudFormDataEntry;
import de.ozgcloud.apilib.vorgang.OzgCloudFormField;
import de.ozgcloud.apilib.vorgang.OzgCloudSubForm;
import de.ozgcloud.collaboration.GrpcFormField;
import de.ozgcloud.collaboration.GrpcSubForm;
import de.ozgcloud.collaboration.GrpcSubFormData;

class CollaborationFormDataMapperTest {

	@Spy
	private final CollaborationFormDataMapper mapper = Mappers.getMapper(CollaborationFormDataMapper.class);

	@DisplayName("From ozgCloudEntries")
	@Nested
	class TestFromOzgCloudEntries {

		private final OzgCloudFormField ozgCloudFormField = OzgCloudFormField.builder().name(FORM_FIELD_NAME).label(FORM_FIELD_LABEL)
				.value(FORM_FIELD_VALUE).build();
		private final OzgCloudSubForm ozgCloudSubForm = OzgCloudSubForm.builder().name(SUB_FORM_NAME).label(SUB_FORM_LABEL)
				.formEntry(OzgCloudFormField.builder().name(SUB_FORM_FIELD_NAME).label(SUB_FORM_FIELD_LABEL).value(SUB_FORM_FIELD_VALUE).build())
				.build();
		private final List<OzgCloudFormDataEntry> ozgCloudEntries = List.of(ozgCloudFormField, ozgCloudSubForm);

		private final FormField formField = FormDataEntryTestFactory.createFormField();
		private final SubForm subForm = FormDataEntryTestFactory.createSubForm();

		@Test
		void shouldMap() {
			doReturn(formField).when(mapper).fromOzgCloudFormField(any());
			doReturn(subForm).when(mapper).fromOzgCloudSubForm(any());

			var entries = mapper.fromOzgCloudEntries(ozgCloudEntries);

			assertThat(entries).containsExactly(formField, subForm);
		}

		@Test
		void shouldHandleNullValue() {
			var entries = mapper.fromOzgCloudEntries(null);

			assertThat(entries).isEmpty();
		}
	}

	@DisplayName("From ozgCloudSubForm")
	@Nested
	class TestFromOzgCloudSubForm {

		private final OzgCloudFormField ozgCloudFormField = OzgCloudFormField.builder()
				.name(FormDataEntryTestFactory.FORM_FIELD_NAME)
				.label(FormDataEntryTestFactory.FORM_FIELD_LABEL)
				.value(FormDataEntryTestFactory.FORM_FIELD_VALUE)
				.build();

		private final OzgCloudFormField ozgCloudSubFormFormField = OzgCloudFormField.builder()
				.name(FormDataEntryTestFactory.SUB_FORM_FIELD_NAME)
				.label(FormDataEntryTestFactory.SUB_FORM_FIELD_LABEL)
				.value(FormDataEntryTestFactory.SUB_FORM_FIELD_VALUE)
				.build();
		private final OzgCloudSubForm ozgCloudSubForm = OzgCloudSubForm.builder()
				.name(FormDataEntryTestFactory.SUB_FORM_NAME)
				.label(FormDataEntryTestFactory.SUB_FORM_LABEL)
				.formEntry(ozgCloudSubFormFormField)
				.build();

		@Test
		void shouldMapFormField() {
			var formField = mapper.fromOzgCloudFormField(ozgCloudFormField);

			assertThat(formField).usingRecursiveComparison().isEqualTo(FormDataEntryTestFactory.createFormField());
		}

		@Test
		void shouldMapSubForm() {
			var subForm = mapper.fromOzgCloudSubForm(ozgCloudSubForm);

			assertThat(subForm).usingRecursiveComparison().isEqualTo(FormDataEntryTestFactory.createSubForm());
		}
	}

	@DisplayName("To grpcFormData")
	@Nested
	class TestToGrpcFormData {

		private final FormField formField = FormDataEntryTestFactory.createFormField();
		private final SubForm subForm = FormDataEntryTestFactory.createSubForm();
		private final List<FormDataEntry> entries = List.of(formField, subForm);

		private final GrpcFormField grpcFormField = GrpcFormDataTestFactory.createFormField();
		private final GrpcSubForm grpcSubForm = GrpcFormDataTestFactory.createSubForm();

		private final GrpcSubFormData grpcSubFormDataFormField = GrpcFormDataTestFactory.createSubFormDataField();
		private final GrpcSubFormData grpcSubFormDataSubForm = GrpcFormDataTestFactory.createSubFormDataForm();

		@BeforeEach
		void mock() {
			doReturn(grpcFormField).when(mapper).toGrpcFormField(any());
			doReturn(grpcSubForm).when(mapper).toGrpcSubForm(any());
		}

		@Test
		void shouldMap() {
			var formData = mapper.toGrpcFormData(entries);

			assertThat(formData.getFormDataList()).containsExactly(grpcSubFormDataFormField, grpcSubFormDataSubForm);
		}
	}

	@DisplayName("To grpc")
	@Nested
	class TestToGrpc {

		@Test
		void shouldMapFormField() {
			var formField = mapper.toGrpcFormField(FormDataEntryTestFactory.createFormField());

			assertThat(formField).usingRecursiveComparison().isEqualTo(GrpcFormDataTestFactory.createFormField());
		}

		@Test
		void shouldMapSubForm() {
			var subForm = mapper.toGrpcSubForm(FormDataEntryTestFactory.createSubForm());

			assertThat(subForm).usingRecursiveComparison().isEqualTo(GrpcFormDataTestFactory.createSubForm());
		}
	}
}