/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.collaboration.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.apilib.vorgang.OzgCloudVorgang;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangService;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangTestFactory;
import de.ozgcloud.collaboration.CollaborationRequest;
import de.ozgcloud.collaboration.CollaborationRequestMapper;
import de.ozgcloud.collaboration.CollaborationRequestTestFactory;
import de.ozgcloud.collaboration.common.callcontext.CollaborationManagerCallContextGrpcClientInterceptor;
import de.ozgcloud.command.CommandTestFactory;
import de.ozgcloud.vorgang.vorgang.GrpcCreateCollaborationRequestData;
import de.ozgcloud.vorgang.vorgang.GrpcCreateCollaborationVorgangRequest;
import de.ozgcloud.vorgang.vorgang.GrpcCreateCollaborationVorgangResponse;
import de.ozgcloud.vorgang.vorgang.VorgangServiceGrpc.VorgangServiceBlockingStub;

class VorgangRemoteServiceTest {

	@Spy
	@InjectMocks
	private VorgangRemoteService service;

	@Mock
	private VorgangServiceBlockingStub vorgangServiceBlockingStub;
	@Mock
	private CollaborationRequestMapper collaborationRequestMapper;
	@Mock
	private CollaborationManagerCallContextGrpcClientInterceptor callContextInterceptor;
	@Mock
	private OzgCloudVorgangService ozgCloudVorgangService;
	@Mock
	private CollaborationVorgangMapper mapper;

	@Nested
	class TestCreateCollaborationVorgang {

		private static final CollaborationRequest REQUEST = CollaborationRequestTestFactory.create();
		private static final GrpcCreateCollaborationVorgangResponse GRPC_RESPONSE = GrpcCreateCollaborationVorgangResponse.newBuilder()
				.setVorgangId(CommandTestFactory.VORGANG_ID).build();

		@Mock
		private GrpcCreateCollaborationVorgangRequest grpcRequest;

		@BeforeEach
		void init() {
			doReturn(vorgangServiceBlockingStub).when(service).getServiceStub();
			when(vorgangServiceBlockingStub.createCollaborationVorgang(any())).thenReturn(GRPC_RESPONSE);
			doReturn(grpcRequest).when(service).buildCreateCollaborationVorgangRequest(any());
		}

		@Test
		void shouldCallBuildCreateCollaborationVorgangRequest() {
			service.createCollaborationVorgang(REQUEST);

			verify(service).buildCreateCollaborationVorgangRequest(REQUEST);
		}

		@Test
		void shouldCallCreateCollaborationVorgang() {
			service.createCollaborationVorgang(REQUEST);

			verify(vorgangServiceBlockingStub).createCollaborationVorgang(grpcRequest);
		}

		@Test
		void shouldReturnVorgangId() {
			var result = service.createCollaborationVorgang(REQUEST);

			assertThat(result).isEqualTo(CommandTestFactory.VORGANG_ID);
		}
	}

	@Nested
	class TestBuildCreateCollaborationVorgangRequest {

		private static final CollaborationRequest REQUEST = CollaborationRequestTestFactory.create();

		@Mock
		private GrpcCreateCollaborationRequestData grpcCollaborationRequest;

		@BeforeEach
		void init() {
			when(collaborationRequestMapper.toGrpcCreateCollaborationRequestData(any())).thenReturn(grpcCollaborationRequest);
		}

		@Test
		void shouldMapCallMapGrpcCreateCollaborationRequestData() {
			service.buildCreateCollaborationVorgangRequest(REQUEST);

			verify(collaborationRequestMapper).toGrpcCreateCollaborationRequestData(REQUEST);
		}

		@Test
		void shouldSetGrpcCollaborationRequest() {
			var result = service.buildCreateCollaborationVorgangRequest(REQUEST);

			assertThat(result.getRequestData()).isSameAs(grpcCollaborationRequest);
		}
	}

	@Nested
	class TestGetServiceStub {

		@Mock
		VorgangServiceBlockingStub serviceStubWithInterceptor;

		@BeforeEach
		void init() {
			when(vorgangServiceBlockingStub.withInterceptors(any())).thenReturn(serviceStubWithInterceptor);
		}

		@Test
		void shouldSetInterceptor() {
			service.getServiceStub();

			verify(vorgangServiceBlockingStub).withInterceptors(callContextInterceptor);
		}

		@Test
		void shouldReturnServiceStub() {
			var result = service.getServiceStub();

			assertThat(result).isSameAs(serviceStubWithInterceptor);
		}
	}

	@Nested
	class TestGetVorgang {

		private static final OzgCloudVorgang OZG_CLOUD_VORGANG = OzgCloudVorgangTestFactory.create();
		private static final Vorgang VORGANG = VorgangTestFactory.create();

		@BeforeEach
		void init() {
			when(ozgCloudVorgangService.getById(any())).thenReturn(OZG_CLOUD_VORGANG);
			when(mapper.fromOzgCloudVorgang(any())).thenReturn(VORGANG);
		}

		@Test
		void shouldCallOzgCloudVorgangService() {
			service.getVorgang(OzgCloudVorgangTestFactory.ID);

			verify(ozgCloudVorgangService).getById(OzgCloudVorgangTestFactory.ID);
		}

		@Test
		void shouldCallMapper() {
			service.getVorgang(OzgCloudVorgangTestFactory.ID);

			verify(mapper).fromOzgCloudVorgang(OZG_CLOUD_VORGANG);
		}

		@Test
		void shouldReturnVorgang() {
			var result = service.getVorgang(OzgCloudVorgangTestFactory.ID);

			assertThat(result).isSameAs(VORGANG);
		}
	}
}